from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanFile, Dataset
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LoadPlmdTicketReplaceOutput(TcBaseObj):
    """
    Output
    
    :var referenceCount: Number of files attached to the Dataset.
    :var ticket: The requested PLMD file transient ticket.
    :var datasetName: Dataset name.
    :var fileNameExt: List of file names extensions.
    :var fileNames: List of file names attached to the Dataset.
    """
    referenceCount: int = 0
    ticket: str = ''
    datasetName: str = ''
    fileNameExt: List[str] = ()
    fileNames: List[str] = ()


@dataclass
class LoadPlmdTicketReplaceResponse(TcBaseObj):
    """
    Response
    
    :var output: A map(Dataset, LoadPlmdTicketReplaceOutput) of input Dataset objects to the required Replace File
    information.
    :var serviceData: Updated Dataset, ImanFile objects and partial errors if any.
    """
    output: DatasetPlmdResponse = None
    serviceData: ServiceData = None


@dataclass
class DatashareManagerReplaceInfo(TcBaseObj):
    """
    Input information about the Dataset and file to be replaced.
    
    :var dataset: The Dataset object to which the referenced ImanFile objects(s) is to be uploaded to.
    :var imanFile: The ImanFile currently referenced by the Dataset. This named reference is intended to be replaced by
    the operation.
    :var namedReferenceName: The name of the reference type relation binding the Dataset to the ImanFile object file.
    This input is used to ensure that the right file is being replaced since Dataset named reference uniqueness is
    based on the named Reference name in addition to the ImanFile.
    :var absoluteFilePath: The absolute file path of the file being uploaded on the client host. This file Path is used
    to generate the "original File name". If empty, an internally generated name is used.
    """
    dataset: Dataset = None
    imanFile: ImanFile = None
    namedReferenceName: str = ''
    absoluteFilePath: str = ''


"""
Map of Dataset and LoadPlmdTicketReplaceOutput.
"""
DatasetPlmdResponse = Dict[Dataset, LoadPlmdTicketReplaceOutput]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BOMWindow, PublishLink, PSBOMView
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindSourceOutput(TcBaseObj):
    """
    Contains source BOMLine and integer based index to point output to corresponding input.
    
    :var inputIndex: Integer pointing to input. Useful to map output with input.
    :var sourceLine: Source BOMLine for input target BOMLine and source BOMWindow. Source BOMLine and Target BOMLine
    are associated via PublishLink.
    """
    inputIndex: int = 0
    sourceLine: BOMLine = None


@dataclass
class FindSourceResponse(TcBaseObj):
    """
    Contains 'FindSourceOutput' containing source BOMLine and index to map to source BOMLine to corresponding input
    target BOMLine.
    
    :var output: 'FindSourceOutput' containing source BOMLine and integer index to map source BOMLine to input target
    BOMLine.
    :var serviceData: 'ServiceData' with plain objects containing source BOMLine and partial errors.
    """
    output: List[FindSourceOutput] = ()
    serviceData: ServiceData = None


@dataclass
class FindTargetsOutput(TcBaseObj):
    """
    Contains target BOMLines and integer based index to point output to corresponding input.
    
    :var inputIndex: Integer pointing to input. Useful to map output with input.
    :var targetLines: Target BOMLine for given source BOMLine and target BOMWindow.Target BOMLine and Source BOMLine
    are associated via PublishLink.
    """
    inputIndex: int = 0
    targetLines: List[BOMLine] = ()


@dataclass
class FindTargetsResponse(TcBaseObj):
    """
    Contains 'FindTargetsOutput' containing target BOMLine objects and index to map to target BOMLine to corresponding
    source.
    
    :var output: 'FindTargetsOutput' containing target BOMLine for input source BOMLine and integer index to map target
    BOMLine to input source BOMLine.
    :var serviceData: 'ServiceData' with plain objects containing target BOMLine objects and partial errors.
    """
    output: List[FindTargetsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetSourceTopLevelOutput(TcBaseObj):
    """
    Contains context PSBOMView of the source of PublishLink for given input target BOMLine. Integer based index points
    PSBOMView to corresponding input.
    
    :var inputIndex: Integer pointing to input. Useful to map output with input.
    :var topLevelBomView: PSBOMView of the source of PublishLink.
    """
    inputIndex: int = 0
    topLevelBomView: PSBOMView = None


@dataclass
class GetSourceTopLevelResponse(TcBaseObj):
    """
    'GetSourceTopLevelResponse' contains vector of 'GetSourceTopLevelOutput' and 'ServiceData'.
    
    :var output: 'GetSourceTopLevelOutput'  containing PSBOMView in which context targets were added and integer index
    to map PSBOMView to input BOMLine.
    :var serviceData: 'ServiceData' containing partial error and PSBOMView in plain objects list.
    """
    output: List[GetSourceTopLevelOutput] = ()
    serviceData: ServiceData = None


@dataclass
class LineAndWindow(TcBaseObj):
    """
    Contains source BOMLine and target BOMWindow in which associated target BOMLine of PublishLink to look for.
    
    :var line: Source BOMLine object.
    :var window: Target BOMWindow object.
    """
    line: BOMLine = None
    window: BOMWindow = None


@dataclass
class LogicallyEquivalentLinesOutput(TcBaseObj):
    """
    Contains logically equivalent BOMLines from input BOMWindow and BOMLine. The integer based index points equivalent
    line to input BOMLine.
    
    :var inputIndex: Integer pointing to input BOMLine. Useful to map output with input.
    :var lines: Logically equivalent BOMLine objects.
    """
    inputIndex: int = 0
    lines: List[BOMLine] = ()


@dataclass
class LogicallyEquivalentLinesResponse(TcBaseObj):
    """
    'LogicallyEquivalentLinesResponse' contains vector of 'LogicallyEquivalentLinesOutput' and 'ServiceData'.
    
    :var output: 'LogicallyEquivalentLinesOutput' with equivalent BOMLine objects  for input BOMLine pointed by integer
    inputIndex.
    :var serviceData: 'ServiceData' with equivalent BOMLines in plain objects and partial errors.
    """
    output: List[LogicallyEquivalentLinesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class PublishDataInfo(TcBaseObj):
    """
    Contains 'PublishLinkInfo' and 'dataFlags'. Each bit of 'dataFlags' denotes what data to be published.
    
    :var linkInfo: 'PublishLinkInfo' containing information to create PublishLink.
    :var dataFlags: 'dataFlags' representing what data to be published.
    """
    linkInfo: PublishLinkInfo = None
    dataFlags: int = 0


@dataclass
class PublishLinkInfo(TcBaseObj):
    """
    Required data to create a PublishLink like name, type, source and target BOMLines.
    
    :var name: object_name to be set on the PublishLink.
    :var type: Valid type to set on the PublishLink's object_type.
    :var source: BOMLine object.
    :var targets: List of BOMLine objects.
    """
    name: str = ''
    type: str = ''
    source: BOMLine = None
    targets: List[BOMLine] = ()


@dataclass
class PublishLinkOutput(TcBaseObj):
    """
    Contains PublishLink created by the operation and integer based index points PublishLink to corresponding input.
    
    :var inputIndex: Integer pointing to input. Useful to map output with input.
    :var publishLink: PublishLink object created during the operation.
    """
    inputIndex: int = 0
    publishLink: PublishLink = None


@dataclass
class PublishLinksResponse(TcBaseObj):
    """
    'PublishLinksResponse' contains list of 'PublishLinkOutput' structures and 'ServiceData'.
    
    :var output: 'PublishLinkOutput' containing PublishLink to which targets were added and integer index to map
    PublishLink to input BOMLine
    :var serviceData: 'ServiceData' with created objects and partial errors.
    """
    output: List[PublishLinkOutput] = ()
    serviceData: ServiceData = None


@dataclass
class SourceAndTargets(TcBaseObj):
    """
    Input structure that contains BOMLine objects representing source and targets of PublishLink
    
    :var source: BOMLine as source for PublishLink object
    :var targets: BOMLine objects as target for PublishLink object
    """
    source: BOMLine = None
    targets: List[BOMLine] = ()


@dataclass
class CompletenessCheckInputData(TcBaseObj):
    """
    Contains string mentioning which action be performed on input BOMLine representing Part.
    
    :var action: string representing action to be performed on BOMLine. Valid values are 'VerifyPartStructInteractive'
    and 'ClearCompletenessCheckResults'. The Completeness check is performed with action string is
    'VerifyPartStructInteractive'. The Completeness check is cleared if string is 'ClearCompletenessCheckResults'.
    :var bomline: BOMLine representing Part structure for which Completeness Check has to be performed.
    """
    action: str = ''
    bomline: BOMLine = None


@dataclass
class CompletenessCheckOutput(TcBaseObj):
    """
    Contains set of Complete, Incomplete and Skipped BOMLines for CompletenessCheck. Integer based index points output
    to corresponding input.
    
    :var inputIndex: Integer pointing to input. Useful to map output with input.
    :var completeLines: BOMLines that satisfy Completeness criteria.
    :var incompleteLines: BOMLines that do not satisfy Completeness criteria.
    :var skippedLines: BOMLines for which Completeness criteria do not apply.
    """
    inputIndex: int = 0
    completeLines: List[BOMLine] = ()
    incompleteLines: List[BOMLine] = ()
    skippedLines: List[BOMLine] = ()


@dataclass
class CompletenessCheckResponse(TcBaseObj):
    """
    'CompletenessCheckResponse' containing list of 'CompletenessCheckOutput' and 'ServiceData'. 
    'CompletenessCheckOutput' contains list of Complete, Incomplete and Skipped BOMLines and integer to map with input
    BOMLine. 'ServiceData' contains any error that might have occurred during operation.
    
    :var output: 'CompletenessCheckOutput' containing Complete, Incomplete and Skipped BOMLine objects. The integer
    inputIndex maps to index of input BOMLine to map output and input.
    :var serviceData: 'ServiceData' contains partial error (if any)
    """
    output: List[CompletenessCheckOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, PSBOMViewRevision, ImanRelation, Dataset, GeneralDesignElement, GeneralDesignElementLink, PSConnection, Participant, Item, PSConnectionRevision, Form, ItemRevision
from enum import Enum
from tcsoa.gen.Core._2006_03.DataManagement import ExtendedAttributes
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GDELinkInfo(TcBaseObj):
    """
    GDE Link Information
    
    :var clientID: A unique string identifier supplied by the caller for each GDELinkInfo. This ID is used to identify
    returned GDELink objects and Partial Errors associated with the input properties.
    :var name: Name of the GDELink to be created.
    :var description: The description text for the GDELink to be created.
    :var type: Type of the GDELink that needs to be created. The default type is GeneralDesignElementInfoLink.
    :var gdeLink: Existing GDELink to update (will be null in case of creation)
    :var attributes: Any additional attribute values to be set on the created GDELink.
    """
    clientID: str = ''
    name: str = ''
    description: str = ''
    type: str = ''
    gdeLink: GeneralDesignElementLink = None
    attributes: List[NameValueStruct] = ()


@dataclass
class GetItemAndRelatedObjectsInfo(TcBaseObj):
    """
    Input structure for 'GetItemAndRelatedObjects'. Must have a 'clientId', must contain an 'itemInfo' that is valid,
    may contain a list of information about revisions to return. If any bvr names are given then the tags for the BVRs
    will be returned.
    
    :var clientId: Client Identifier generated by client to identify output
    :var itemInfo: Item specifications, the itemInfo input is valid when one of the following is true:
     - has the uid field set to the UID of an existing Item,
     - has the ids filed set to key attribute values which identify a single existing Item,
     - it may be blank if the RevInfo uid field is set to the UID of an existing ItemRevision.
    :var revInfo: Revision specifications
    :var datasetInfo: Filter used to select datasets to return.
    :var bvrTypeNames: A list of a BVRs to return. optional.
    """
    clientId: str = ''
    itemInfo: ItemInfo = None
    revInfo: RevInfo = None
    datasetInfo: DatasetInfo = None
    bvrTypeNames: List[str] = ()


@dataclass
class GetItemAndRelatedObjectsItemOutput(TcBaseObj):
    """
    This structure contains item information.
    
    :var clientId: Identifier of input data used to generate output.
    :var item: Item object.
    :var itemRevOutput: List of 'RevisionOutputs'. When multiple are returned, the order will be oldest first to latest
    last.
    """
    clientId: str = ''
    item: Item = None
    itemRevOutput: List[RevisionOutput] = ()


@dataclass
class GetItemAndRelatedObjectsResponse(TcBaseObj):
    """
    Return structure for 'GetItemAndRelatedObjects' operation
    
    :var output: List of 'GetItemAndRelatedObjectsItemOutput'
    :var serviceData: Standard ServiceData member
    """
    output: List[GetItemAndRelatedObjectsItemOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetNRPatternsWithCountersResponse(TcBaseObj):
    """
    Response structure for getNRPatternsWithCounters service operation.
    
    :var patterns: List of structs which contain Patterns which has counters.
    :var preferredPattern: List of preferred patterns.
    :var condition: List of conditions.
    :var serviceData: Standard ServiceData structure. It contains partial errors and failures along with the clientIds.
    """
    patterns: List[PatternsWithCounters] = ()
    preferredPattern: List[str] = ()
    condition: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class GetNextIdsResponse(TcBaseObj):
    """
    Response structure for 'getNextIds' service operation.
    
    :var nextIds: List containing next Id strings generated as per the pattern.
    :var serviceData: Standard 'ServiceData' structure. It contains partial errors and failures along with the
    clientIds.
    """
    nextIds: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class GetRevNRAttachResponse(TcBaseObj):
    """
    Response structure for getRevNRAttachDetails service operation.
    
    :var initRevDetails: List containing information about Initial Revision Type.
    :var secRevDetails: List containing information about Secondary Revision Type.
    :var supplRevDetails: List containing information about Supplemental Revision Type.
    :var excludedLetters: List of exluded letters.
    :var serviceData: Standard ServiceData structure. It contains partial errors and failures along with the clientIds.
    """
    initRevDetails: List[RevOptionDetails] = ()
    secRevDetails: List[RevOptionDetails] = ()
    supplRevDetails: List[RevOptionDetails] = ()
    excludedLetters: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class InfoForNextId(TcBaseObj):
    """
    Input structure for 'getNextIds' operation. It contains information about the object type, property name and
    pattern to be used for id generation.
    
    :var typeName: Type Name
    :var propName: Property Name
    :var pattern: Pattern String
    """
    typeName: str = ''
    propName: str = ''
    pattern: str = ''


@dataclass
class ItemElementProperties(TcBaseObj):
    """
    Input structure for createOrUpdateItemElements operation
    
    :var clientId: Identifier that helps the client to track the object(s) created should be unique for the input set
    :var name: Name of the ItemElement to be created
    :var type: Type of the ItemElement to be created. If it is not passed, operation takes default type.
    :var description: Description of the ItemElement to be created
    :var itemElemAttributes: Map<String,String> containing the attributes to be modified and its values. Required in
    case of updating an existing GDE.
    :var itemElement: ItemElement to be updated.
    """
    clientId: str = ''
    name: str = ''
    type: str = ''
    description: str = ''
    itemElemAttributes: AttributeMap = None
    itemElement: BusinessObject = None


@dataclass
class ItemElementsOutput(TcBaseObj):
    """
    This structure contains information for 'createOrUpdateItemElements' operation.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var itemElem: ItemElement object that was created
    """
    clientId: str = ''
    itemElem: GeneralDesignElement = None


@dataclass
class ItemInfo(TcBaseObj):
    """
    This structure contains either an item id or uid. Has a flag that indicates if the id is to be used first. If set
    then the id will be used. If the id does not resolve to an item then the uid will be used. If the flag is not set
    then the uid will be used first. If the uid does not resolve to an item then the id will be used. If no matching
    item is found then a null entry is placed in the output list and an error is returned.
    
    :var clientId: Client Identifier generated by client to identify output
    :var useIdFirst: A flag to indicate if the id is to be used before the uid.
    :var uid: Item uid string for Item to retrieve.
    :var ids: List of attributes and values for selecting the item. For pre TC8.1 versions only one entry is supported.
    This entry has the name 'item'_id.
    """
    clientId: str = ''
    useIdFirst: bool = False
    uid: str = ''
    ids: List[AttrInfo] = ()


@dataclass
class AttrInfo(TcBaseObj):
    """
    This structure contains information for specifying name value pairs.
    
    :var name: The name of the attribute.
    :var value: The value of the attribute.
    """
    name: str = ''
    value: str = ''


@dataclass
class MasterFormPropertiesInfo(TcBaseObj):
    """
    The data structure holds information for specifying property values to be set on a Form.
    
    :var form: Reference to Form object
    :var propertyValueInfo: A list of propertie names and values
    """
    form: Form = None
    propertyValueInfo: List[PropertyNameValueInfo] = ()


@dataclass
class NRAttachInfo(TcBaseObj):
    """
    NRAttachInfo structure contains information about the object type and property name for which Naming Rule
    information needs to be obtained.
    
    :var typeName: Type Name
    :var propName: Property Name
    """
    typeName: str = ''
    propName: str = ''


@dataclass
class NROutput(TcBaseObj):
    """
    This structure contains named reference information.
    
    :var namedReference: Object referenced
    :var namedReferenceName: Named Reference name
    :var ticket: Read ticket if referenced file and if requested.
    """
    namedReference: BusinessObject = None
    namedReferenceName: str = ''
    ticket: str = ''


@dataclass
class NameValueStruct(TcBaseObj):
    """
    This structure contains property name and value pairs for each property.
    
    :var name: Title of the attribute like time, act_location that needs to be set
    :var values: Values of the attribute to be set
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class NamedReferenceFilter(TcBaseObj):
    """
    This structure contains information for determining which datasets to consider to return. This specifies a list of
    named references and the UID referenced by the named reference.
    
    :var namedReference: The name of the named reference.
    :var uidReferenced: The unique identifier referenced.
    """
    namedReference: str = ''
    uidReferenced: str = ''


@dataclass
class NamedReferenceList(TcBaseObj):
    """
    This structure contains information determining which named reference information to return. This specifies a named
    reference to return and a flag to specify, if the named reference is to a file, to return an FMS read ticket.
    
    :var namedReference: Named Reference to return.
    :var ticket: If this Named Reference is to a file, include an FMS read ticket if set.
    """
    namedReference: str = ''
    ticket: bool = False


@dataclass
class ParticipantInfo(TcBaseObj):
    """
    This structure contains information about new participant to be created or to find an existing one.
    
    :var assignee: The participant which can be a User, GroupMember or a ResourcePool
    :var participantType: Participant type to be assigned or added.  List of Participant types provided are:
    - ProposedResponsibleParty
    - ProposedReviewer
    - ChangeImplementationBoard
    - ChangeReviewBoard
    - ChangeSpecialist1
    - ChangeSpecialist2
    - ChangeSpecialist3
    - Analyst
    - Requestor
    
    
    
    Custom participant types are also allowed.
    :var clientId: A unique string supplied by the caller
    """
    assignee: BusinessObject = None
    participantType: str = ''
    clientId: str = ''


@dataclass
class Participants(TcBaseObj):
    """
    'Participants' structure contains the participants to be added / removed and object to which participants are to be
    added or removed.
    
    :var itemRev: Item Revision to which the Participant objects are added/removed.
    :var participant: List of Participant objects to be added/removed.
    """
    itemRev: ItemRevision = None
    participant: List[Participant] = ()


@dataclass
class PatternsWithCounters(TcBaseObj):
    """
    This structure holds patterns with counters.
    
    :var patternStrings: A list of pattern strings with counters
    """
    patternStrings: List[str] = ()


@dataclass
class PropertyNameValueInfo(TcBaseObj):
    """
    This structure contains information for specifying property name value pairs.
    
    :var propertyName: Name of the property
    :var propertyValues: A list of value(s) of the property
    """
    propertyName: str = ''
    propertyValues: List[str] = ()


@dataclass
class BOHierarchy(TcBaseObj):
    """
    This structure contains information for a bottom up hierarchy starting with a Business Object name and going up the
    hierarchy of parents up to the primary Business Object.
    
    :var BOName: Business Object Name
    :var BOParents: Ordered list of Business Object parents from bottom up
    """
    BOName: str = ''
    BOParents: List[str] = ()


@dataclass
class RelatedObjectInfo(TcBaseObj):
    """
    The data structure for specifying related object information.
    
    :var relatedObject: Object related to the new Item or IitemRevision.
    :var action: An integer representing the action that specifies the why the relatedObject as the other side object
    while applying the deep copy rules. Valid values are : CopyAsObjectType = 0, (A new copy of the object represented
    by otherSideObjectTag will be created and while propagatring the relations) CopyAsRefType = 1, (The same object
    represented by otherSideObjectTag will be used to propagate the relations) DontCopyType =2, (The relation will not
    be propagated.) RelateToLatest = 3, (If the other side object is an ItemRevision, then the latest version of the
    revision will be used while propagating the relation) ReviseAndRelateToLatest = 4 (If the other side object is an
    Item Revision, it will be revised and the newly created revision will be used while propagating the relations)
    :var isSecondary: Flag indicating whether the 'relatedObject' is participating as a secondary object in the
    relation.
    """
    relatedObject: BusinessObject = None
    action: int = 0
    isSecondary: bool = False


@dataclass
class BOWithExclusionIn(TcBaseObj):
    """
    Structure to hold the information about Business Object name and its exclusion list.
    
    :var boTypeName: Primary Business Object name for which hierarchies of displayable Business Objects are returned.
    :var exclusionBOTypeNames: List of Business Object names to be excluded from the returned list.
    """
    boTypeName: str = ''
    exclusionBOTypeNames: List[str] = ()


@dataclass
class BVROutput(TcBaseObj):
    """
    This structure contains bom view revision information.
    
    :var bvr: The BVR associated with the ItemRevision if requested.
    :var viewTypeName: The bvr view type.
    """
    bvr: PSBOMViewRevision = None
    viewTypeName: str = ''


@dataclass
class RevInfo(TcBaseObj):
    """
    This structure contains information to determine the revisions to return.
    
    :var clientId: Client Identifier generated by client to identify output.
    :var processing: Specifies what data to use to determine the revisions to return.
    The legal values are: 
    - All - All the revisions of the input Item are returned
    - None - No revisions are returned.
    - Nrev - Number of revisions based on the 'RevInfo::nRevs' input.
    - Min - The latest revision.
    - Rule - The revision as configured by the 'RevInfo::revisionRule' input.
    - Ids - The revision based on the 'RevInfo::Id' input.
    
    
    :var useIdFirst: A flag to indicate if the id is to be used before the uid.
    :var uid: Revision uid for revision to retrieve.
    :var id: Revision id for revision to retrieve.
    :var nRevs: The number of revisions to retrieve, or the number of existing revisions, whichever is smaller. The
    revisions returned will be the latest ones. The first one returned is the oldest of the group. The last the latest.
    :var revisionRule: The revision rule to be used.
    """
    clientId: str = ''
    processing: Processing = None
    useIdFirst: bool = False
    uid: str = ''
    id: str = ''
    nRevs: int = 0
    revisionRule: str = ''


@dataclass
class RevOptionDetails(TcBaseObj):
    """
    This structure contain revision option details.
    
    :var revOption: The next available revision option
    :var revFormat: Format specified for revision type.
    """
    revOption: str = ''
    revFormat: int = 0


@dataclass
class ReviseInfo(TcBaseObj):
    """
    The data structure contains the information required to create the new revision.  If 'DeepCopyInfo' is supplied it
    overrides the existing relations on the existing ItemRevision.
    
    :var clientId: Identifier to uniquely identify the input
    :var baseItemRevision: Original ItemRevision object to be used for the revise operation
    :var newRevId: ItemRevision id string for creating new ItemRevision object
    :var name: Name string for creating new Item/ItemRevision
    :var description: Description string for creating new Item/ItemRevision
    :var deepCopyInfo: List of 'DeepCopyData' objects to be used for propagating the relations of the ItemRevision. If
    this list is supplied, then only the relations listed in this list will be carried forward to the new ItemRevision.
    No other Deep Copy Rules defined in the system will get applied. Thus the 'DeepCopyData' in this list overrides
    everything defined in the system to carry forward the relations of the ItemRevision.
    :var newItemRevisionMasterProperties: The information for the new ItemRevisionMaster Form property values
    """
    clientId: str = ''
    baseItemRevision: ItemRevision = None
    newRevId: str = ''
    name: str = ''
    description: str = ''
    deepCopyInfo: List[DeepCopyData] = ()
    newItemRevisionMasterProperties: MasterFormPropertiesInfo = None


@dataclass
class ReviseOutput(TcBaseObj):
    """
    This structure contains newly created ItemRevision object and related objects.
    
    :var newItemRev: Newly created ItemRevision object.
    :var relatedObjects: A list of 'RelatedObjectInfo' which represents the new objects created or the existing the
    objects used to propagate the relations during the deep copy.
    """
    newItemRev: ItemRevision = None
    relatedObjects: List[RelatedObjectInfo] = ()


@dataclass
class ReviseResponse2(TcBaseObj):
    """
    Return structure for revise operation.
    
    :var reviseOutputMap: Map whose keys are the input clientIds and output values pairs ('string', 'ReviseOutput').
    :var serviceData: This contains the status of the operation.
    """
    reviseOutputMap: ReviseOutputMap = None
    serviceData: ServiceData = None


@dataclass
class RevisionOutput(TcBaseObj):
    """
    Output structure containing revision information
    
    :var clientId: Identifier of input data used to generate output.
    :var itemRevision: ItemRevision object.
    :var bvrs: The BVRs associated with the item revision if requested.
    :var datasetOutput: List of 'DatasetOutput'.
    """
    clientId: str = ''
    itemRevision: ItemRevision = None
    bvrs: List[BVROutput] = ()
    datasetOutput: List[DatasetOutput] = ()


@dataclass
class SaveAsNewItemInfo(TcBaseObj):
    """
    This data structure contain existing and new object info for save as new Item operation.
    
    :var clientId: Identifier to uniquely identify the input
    :var baseItemRevision: Original ItemRevision to be used for the 'ItemRevisionSaveAs' operation
    :var newItemId: Item id string for creating new Item
    :var newRevId: ItemRevision id string for creating new ItemRevision
    :var name: Name string for creating new Item/ItemRevision.
    :var description: Description string for creating new Item/ItemRevision
    :var deepCopyInfo: List of deep copy data to be used for propagating the relations of the ItemRevision. If this
    list is supplied, then only the relations listed in this list will be carried forward to the new ItemRevision. No
    other Deep Copy Rules defined in the system will get applied. Thus the 'DeepCopyData' in this list overrides
    everything defined in the system to carry forward the relations of the ItemRevision.
    :var newItemMasterProperties: The properties to be set for the newly created master form of the new Item
    :var newItemRevisionMasterProperties: The properties to be set for the newly created master form of the new
    ItemRevision
    """
    clientId: str = ''
    baseItemRevision: ItemRevision = None
    newItemId: str = ''
    newRevId: str = ''
    name: str = ''
    description: str = ''
    deepCopyInfo: List[DeepCopyData] = ()
    newItemMasterProperties: MasterFormPropertiesInfo = None
    newItemRevisionMasterProperties: MasterFormPropertiesInfo = None


@dataclass
class SaveAsNewItemOutput2(TcBaseObj):
    """
    The data strucutre contains newly created object inforamtiobn.
    
    :var newItem: Newly created Item object
    :var newItemRev: Newly created ItemRevision object
    :var relatedObjects: List of newly created related objects
    """
    newItem: Item = None
    newItemRev: ItemRevision = None
    relatedObjects: List[RelatedObjectInfo] = ()


@dataclass
class SaveAsNewItemResponse2(TcBaseObj):
    """
    The data structure contains return information for the operation.
    
    :var saveAsOutputMap: Map whose keys are the input clientIds and output values pairs('string',
    'SaveAsNewItemOutput2')
    :var serviceData: Service data
    """
    saveAsOutputMap: SaveAsNewItemOutputMap = None
    serviceData: ServiceData = None


@dataclass
class SecondaryData(TcBaseObj):
    """
    This structure contains information for secondary object data.
    
    :var clientId: identifier defined by user to track the related object.
    :var secondary: the secondary object pointed to by the relation
    :var userData: Object associated with the relation to the above secondary object
    :var properties: Additional attributes to be set on the relation. These are meant for any derived classes of
    TCRelation with additional attributes.
    """
    clientId: str = ''
    secondary: BusinessObject = None
    userData: BusinessObject = None
    properties: List[NameValueStruct] = ()


@dataclass
class TypeAndItemRevInfo(TcBaseObj):
    """
    'TypeAndItemRevInfo' structure contains information about the object type and revision for which Revision Naming
    Rule information needs to be obtained.
    
    :var typeName: Type name of Item or ItemRevision.
    :var itemRev: ItemRevision object selected.
    """
    typeName: str = ''
    itemRev: ItemRevision = None


@dataclass
class ConnectionOutput(TcBaseObj):
    """
    This structure contains information for 'createConnections' operation.
    
    :var clientId: Identifying string from the source ConnectionProperties.
    :var connection: The newly created Connection object
    :var connectionRev: The initial revision for the newly created ConnectionRevision
    """
    clientId: str = ''
    connection: PSConnection = None
    connectionRev: PSConnectionRevision = None


@dataclass
class ConnectionProperties(TcBaseObj):
    """
    Input structure for createConnections operation.  Specifies attributes for the new Connection or ConnectionRevision.
    
    :var clientId: A unique string supplied by the caller for each ConnectionProperties. This ID is used to identify
    returned ConnectionOutput elements and Partial Errors associated with the input ConnectionProperties, optional
    :var connId: The ID of the Connection to be created, optional
    :var name: The name of the Connection to be created, optional
    :var type: The type of the Connection to be created. The default type is Connection
    :var revId: The ID of the initial revision of the Connection to be created, optional
    :var description: The description text for the Connection to be created, optional
    :var extendedAttributes: Any additional attribute values to be set on the created Connection, ConnectionRevision or
    coresponding Master Forms, optional
    """
    clientId: str = ''
    connId: str = ''
    name: str = ''
    type: str = ''
    revId: str = ''
    description: str = ''
    extendedAttributes: List[ExtendedAttributes] = ()


@dataclass
class CreateConnectionsResponse(TcBaseObj):
    """
    Return structure for createConnections operation
    
    :var serviceData: Standard ServiceData member
    :var output: A list of created Connection objects and associate ConnectionRevision in the form of ConnectionsOutput
    structure.
    """
    serviceData: ServiceData = None
    output: List[ConnectionOutput] = ()


@dataclass
class CreateIn(TcBaseObj):
    """
    The data structure contains data for 'creation' operation.
    
    :var clientId: Unique client identifier. If there is an error in create, the client id in the ServiceData
    PartialError list can be used to associate to the clientId in the input to detect which object creation failed.
    :var data: Input data for create operation. This is essentially a map of property name-value pairs representing the
    input data for creation. It also is recursive referencing secondary input data (e.g For Item creation, the primary
    input data is for Item and secondary input data is for Item Master and ItemRevision).
    """
    clientId: str = ''
    data: CreateInput = None


@dataclass
class CreateInput(TcBaseObj):
    """
    This structure captures the inputs required for creation of a business object. This is a recursive structure
    containing the 'CreateInput'(s) for any secondary(compounded) objects that might be created (e.g Item also creates
    ItemRevision and ItemMaster Form, etc.)
    
    :var boName: Business object type name
    :var stringProps: Map of string property names to values ('string, string')
    :var boolArrayProps: Initial values of properties of type bool array. A map of property names to values
    (string/list of bools).
    :var dateProps: Map of DateTime property names to values ('string, 'date)
    :var dateArrayProps: Initial values of properties of type DateTime array. A map of property names to values
    (string/list of DateTimes).
    :var tagProps: Map of BusinessObject property names to values ('string, BusinessObjec't)
    :var tagArrayProps: Initial values of properties of type BusinessObject array. A map of property names to values
    (string/list of BusinessObjects)
    :var compoundCreateInput: Initial secondary CreateInput values of reference or relation properties. A map of
    reference or reation property names to values (string/list of CreateInputs).
    :var stringArrayProps: Initial values of properties of type string array. A map of property names to values
    (string/list of strings).
    :var doubleProps: Map of double property names to values ('string, double')
    :var doubleArrayProps: Initial values of properties of type double array. A map of property names to values
    (string/list of doubles).
    :var floatProps: Map of float property names to values ('string, float')
    :var floatArrayProps: Initial values of properties of type float array. A map of property names to values
    (string/list of floats).
    :var intProps: Map of int property names to values ('string, int')
    :var intArrayProps: Initial values of properties of type int array. A map of property names to values (string/list
    of ints).
    :var boolProps: Map of boolean property names to values (string, bool)
    """
    boName: str = ''
    stringProps: StringMap = None
    boolArrayProps: BoolVectorMap = None
    dateProps: DateMap = None
    dateArrayProps: DateVectorMap = None
    tagProps: TagMap = None
    tagArrayProps: TagVectorMap = None
    compoundCreateInput: CreateInputMap = None
    stringArrayProps: StringVectorMap = None
    doubleProps: DoubleMap = None
    doubleArrayProps: DoubleVectorMap = None
    floatProps: FloatMap = None
    floatArrayProps: FloatVectorMap = None
    intProps: IntMap = None
    intArrayProps: IntVectorMap = None
    boolProps: BoolMap = None


@dataclass
class CreateOrUpdateGDELinksResponse(TcBaseObj):
    """
    Create Or Update GDE Links Response
    
    :var successfullyProcessedGDELinks: A map containing the clientIds and the successfully created/updated
    GeneralDesignElementLink objects
    :var serviceData: Service data to report partial errors.
    """
    successfullyProcessedGDELinks: SuccessfullyProcessedGDELinksMap = None
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateItemElementsResponse(TcBaseObj):
    """
    Return structure for createOrUpdateItemElements operation
    
    :var serviceData: Standard ServiceData member containing created/updated objects and partial errors if any
    :var output: A list of createItemElementsOutput
    """
    serviceData: ServiceData = None
    output: List[ItemElementsOutput] = ()


@dataclass
class CreateOrUpdateRelationsInfo(TcBaseObj):
    """
    The required parameters to create the relation between the primary and secondary object.
    
    :var clientId: Identifier defined by user to track the related object.
    :var relationType: Realtion of interest, required for create or update.
    :var primaryObject: The primary object of interest, required for create or update.
    :var secondaryData: List of secondary objects per primaryObject via relation of type 'relationType'.
    :var relations: Instead of secondary business object in 'secondaryData' and 'primaryObject', this can be used, if
    such an object is available. This is only used if the 'primaryObject' is not specified. If 'relations' is
    specified, the primaries for all of them must be the same and the 'relations' must have the same 'relationType'.
    """
    clientId: str = ''
    relationType: str = ''
    primaryObject: BusinessObject = None
    secondaryData: List[SecondaryData] = ()
    relations: List[ImanRelation] = ()


@dataclass
class CreateOrUpdateRelationsOutput(TcBaseObj):
    """
    This structure contains information for 'CreateOrUpdateRelationsOutput'.
    
    :var clientId: The unmodified value from the 'CreateOrUpdateRelationsInfo.clientId'. This can be used by the caller
    to indentify this data structure with the source input data
    :var relations: The newly created relation.
    """
    clientId: str = ''
    relations: List[ImanRelation] = ()


@dataclass
class CreateOrUpdateRelationsResponse(TcBaseObj):
    """
    The relations created based on the input primary, secondary and relation type information and the errors that might
    have been encountered while creating, updating the relations.
    
    :var output: A list of 'CreateOrUpdateRelationsOutput'
    :var serviceData: Standard 'ServiceData' object to hold the partial errors that the operation encounters.
    """
    output: List[CreateOrUpdateRelationsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOut(TcBaseObj):
    """
    This structure contains information or create operation including unique client identifier.
    
    :var clientId: Unique client identifier
    :var objects: A list of objects that were created
    """
    clientId: str = ''
    objects: List[BusinessObject] = ()


@dataclass
class CreateResponse(TcBaseObj):
    """
    Response object for create operation containing vector of objects that were created and partial errors.
    
    :var output: Vector of output objects representing objects that were created.
    :var serviceData: Service Data including partial errors that are mapped to the client id from the input. Created
    objects are also added to the created objects list in the Service Data.
    """
    output: List[CreateOut] = ()
    serviceData: ServiceData = None


@dataclass
class AddParticipantInfo(TcBaseObj):
    """
    'AddParticipantInfo' structure is an input for the 'addParticipants' operation.
    
    :var itemRev: An item revision to add participants to.
    :var participantInfo: Contains information to create a new participant or find an existing one.
    """
    itemRev: ItemRevision = None
    participantInfo: List[ParticipantInfo] = ()


@dataclass
class AddParticipantOutput(TcBaseObj):
    """
    'AddParticipantOutput' structure contains Item Revision and the added participants along with Service Data.
    
    :var participantOutput: List of structures containing the participants and the object to which participants are
    added.
    :var serviceData: The service data.
    """
    participantOutput: List[Participants] = ()
    serviceData: ServiceData = None


@dataclass
class DatasetFilter(TcBaseObj):
    """
    The structure contains information for determining which datasets to return.
    
    :var useNameFirst: Filter with the 'datasetName' first.
    :var processing: Specifies which version or versions of the Dataset are returned.
     Legal values are: 
    - All - All versions of the dataset are returned.
    - Min - The version 0 of the dataset is returned.
    - None - No dataset is returned.
    
    
    :var nrFilters: Filter to select datasets with a particular named reference to a specific uid. If empty then not
    used in filtering.
    :var name: The name of the Dataset objects to return. If empty then not used in filtering.
    :var relationFilters: The relation filters to use.
    """
    useNameFirst: bool = False
    processing: Processing = None
    nrFilters: List[NamedReferenceFilter] = ()
    name: str = ''
    relationFilters: List[DatasetRelationFilter] = ()


@dataclass
class DatasetInfo(TcBaseObj):
    """
    Input structure that the information for determining which datasets to return. If the 'datasetUid' is specified,
    then only that dataset will be returned. If the dataset does not exist then a null dataset is returned. If the
    'datasetUid' is not specified then the dataset filter determines the datasets to return. For each dataset to be
    returned, the 'NamedRef' will determine the named reference information to return. For each named reference
    returned that is a reference to a file, an FMS read ticket will be returned if 'ticket' is set.
    
    :var clientId: A unique string used to identify return data elements and partial errors associated with this input
    structure.
    :var uid: The uid of the only Dataset to return.
    :var filter: If a UID is not specified, determines the Dataset objects to return.
    :var namedRefs: Information about named references to return for each Dataset.
    """
    clientId: str = ''
    uid: str = ''
    filter: DatasetFilter = None
    namedRefs: List[NamedReferenceList] = ()


@dataclass
class DatasetOutput(TcBaseObj):
    """
    Output structure for Dataset information
    
    :var clientId: A unique string used to identify return data elements and partial errors associated with this input
    structure.
    :var dataset: Dataset object .
    :var relationTypeName: Relationship type to the dataset.
    :var namedReferenceOutput: The list of 'NROutput' for the dataset if requested.
    """
    clientId: str = ''
    dataset: Dataset = None
    relationTypeName: str = ''
    namedReferenceOutput: List[NROutput] = ()


@dataclass
class DatasetProperties2(TcBaseObj):
    """
    Input structure for createDatasets operation
    
    :var clientId: Identifier that helps the client to track the object(s) created, required; should be unique for the
    input set
    :var type: Type of the Dataset to be created
    :var name: Name of the Dataset to be created
    :var description: Description of the Dataset to be created, may be an empty string.
    :var toolUsed: Name of the Tool used to open the created Dataset, may be an empty string.  If it is an empty string
    no Tool will be associated to the Dataset to be created.
    :var datasetId: Unique identifier for the Dataset, may be an empty string. If the preference AE_datase_id_usage is
    ON, the id will be generated through user exit mechanism.
    :var datasetRev: The revision of this Dataset, may be an empty string.  If the preference AE_datase_id_usage is ON,
    the rev will be generated through user exit mechanism.
    :var container: Object reference of the container used to hold the created dataset, optional
    :var relationType: Name of the relation type for the Dataset to be created, may be an empty string.
    """
    clientId: str = ''
    type: str = ''
    name: str = ''
    description: str = ''
    toolUsed: str = ''
    datasetId: str = ''
    datasetRev: str = ''
    container: BusinessObject = None
    relationType: str = ''


@dataclass
class DatasetRelationFilter(TcBaseObj):
    """
    This structure contains information for determining which datasets to consider to return. The relation type and
    Dataset type will be used to produce a set of datasets.
    
    :var relationTypeName: The relation type name specifies the relation that relates the ItemRevision to the Dataset.
    If not specified then all.
    :var datasetTypeName: The type name of Dataset to return. If empty then all.
    """
    relationTypeName: str = ''
    datasetTypeName: str = ''


@dataclass
class DeepCopyData(TcBaseObj):
    """
    The data structure holds information required to apply deep copy rules.
    
    :var otherSideObjectTag: Object on which the deep copy should be performed.
    :var relationTypeName: Name of the relation that need to be deep copied.
    :var newName: New name for the new copy of the object represented by the 'otherSideObject'. The value for the
    'newName' will be null if 'action' is not 'CopyAsObject' or 'ReviseAndRelateToLatest'.
    :var action: Integer representing the action to be performed on the object represented by ''otherSideObjectTag''. 
    The values for action are: 
    - CopyAsObjectType = 0
    - CopyAsRefType = 1
    - DontCopyType =2
    - RelateToLatest = 3
    - ReviseAndRelateToLatest = 4
    
    
    :var isTargetPrimary: Indicates whether the given ItemRrevision is a primary object in the relation to be deep
    copied.
    :var isRequired: Indicates whether the deep information is from a mandatory deepcopy rule
    :var copyRelations: Indicates whether the properties on the relation should be copied
    """
    otherSideObjectTag: BusinessObject = None
    relationTypeName: str = ''
    newName: str = ''
    action: int = 0
    isTargetPrimary: bool = False
    isRequired: bool = False
    copyRelations: bool = False


@dataclass
class DisplayableBOsOut(TcBaseObj):
    """
    This structure contains a list of displayable business(BO) object under a given BO (displayable sub-BO hierarchy).
    
    :var boTypeName: Business object name for which displayable hierarchy is returned
    :var displayableBOTypeNames: Hierarchy of displayable business Objects
    """
    boTypeName: str = ''
    displayableBOTypeNames: List[BOHierarchy] = ()


@dataclass
class DisplayableSubBOsResponse(TcBaseObj):
    """
    Response for getDisplayableBusinessObjects
    
    :var output: Vector of output objects representing displayable types during create of a BO
    :var serviceData: Service data including partial errors that are mapped to the input BO type name
    """
    output: List[DisplayableBOsOut] = ()
    serviceData: ServiceData = None


class Processing(Enum):
    """
    The processing of information. Legal values are: 'None,  All,  Min, Ids, Rule and  Nrev'.
    """
    None_ = 'None'
    All = 'All'
    Min = 'Min'
    Ids = 'Ids'
    Rule = 'Rule'
    Nrev = 'Nrev'


"""
Map of reference or relation property name to secondary 'CreateInput' objects ('string, vector<CreateInput>').
"""
CreateInputMap = Dict[str, List[CreateInput]]


"""
Map of date property names to values ('string, date').
"""
DateMap = Dict[str, datetime]


"""
Map of DateTime array property names to values ('string, vector<DateTime>').
"""
DateVectorMap = Dict[str, List[datetime]]


"""
Map of double property names to values ('string, double').
"""
DoubleMap = Dict[str, float]


"""
Map of double array property names to values ('string, vector<double>').
"""
DoubleVectorMap = Dict[str, List[float]]


"""
Map of float property names to values ('string, float').
"""
FloatMap = Dict[str, float]


"""
Map of float array property names to values ('string, vector<float>').
"""
FloatVectorMap = Dict[str, List[float]]


"""
A map of attribute name to value pairs.
"""
AttributeMap = Dict[str, str]


"""
Map of int property names to values ('string, int').
"""
IntMap = Dict[str, int]


"""
Map of int array property names to values ('string, vector<int>').
"""
IntVectorMap = Dict[str, List[int]]


"""
A map of 'ReviseOutput' ('string, ReviseOutput').
"""
ReviseOutputMap = Dict[str, ReviseOutput]


"""
A map of the clientId from the input data to the output  from the service ('string, SaveAsNewItemOutput2')
"""
SaveAsNewItemOutputMap = Dict[str, SaveAsNewItemOutput2]


"""
Map of bool property names to values '(string, bool').
"""
BoolMap = Dict[str, bool]


"""
Map of string property names to values ('string, string').
"""
StringMap = Dict[str, str]


"""
Map of string array property names to values ('string, vector<string>').
"""
StringVectorMap = Dict[str, List[str]]


"""
Map of bool array property names to values ('string, vector< bool >').
"""
BoolVectorMap = Dict[str, List[bool]]


"""
A map of successfully processed GDE link to GeneralDesignElementLink.
"""
SuccessfullyProcessedGDELinksMap = Dict[str, GeneralDesignElementLink]


"""
Map of BusinessObject property names to values ('string, BusinessObject').
"""
TagMap = Dict[str, BusinessObject]


"""
Map of BusinessObject array property names to values ('string, vector<BusinessObject>').
"""
TagVectorMap = Dict[str, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awp0XRTObjectSet, POM_object
from tcsoa.gen.Internal.AWS2._2013_12.DataManagement import GetStyleSheetDatasetInfo, GetStyleSheetClassificationData
from typing import Dict, List
from tcsoa.gen.Internal.AWS2._2012_10.DataManagement import ObjectLsdInfo, NameValueStruct
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetInitialTableRowDataResponse(TcBaseObj):
    """
    Structure containing  list of rows on which all proposed property values are populated. This information can be
    used finally to create a row to the table property.
    
    :var tableRows: List of property information which will have property values pre-populated.
    """
    tableRows: List[TableRow] = ()


@dataclass
class GetStyleSheetIn(TcBaseObj):
    """
    Input for the getStyleSheet operation.
    
    :var targetPage: The page in the XRT stylesheet to process. This field is ignored if the parameter processEntireXRT
    is set to true. This value may be an empty string which would cause the server to process the first page in the
    XRT, or it may be a titleKey for a page in the XRT, for example web_xrt_Overview. If the target page is not found,
    then the first page in the XRT is processed.
    :var businessObject: The businessObject for which to retrieve a stylesheet. This field may be NULL. Typically it
    would only be NULL in the case of CREATE.
    :var businessObjectType: The Teamcenter data type for which to retrieve a stylesheet, if the businessObject
    parameter is NULL, then this field must be populated, otherwise it is ignored.
    :var styleSheetType: The type of stylesheet to return. Legal values are: SUMMARY, CREATE, RENDERING, or INFO.
    :var styleSheetLastModDate: The last save date of the stylesheet.
    :var clientContext: The map of client context key value entries, where the key is the client and the location in
    the client, and the value is the unique id for the data being presented in that panel. These values are used to
    process visibleWhen clasuses in the XRT, pages may be enabled or disabled based on where it is being presented in
    the client. If the isRedLineMode key value is true and source object is of type ItemRevision then it is compared
    against configured revision. If configured revision is not found then source object is compared to previous
    revision.
    
    For example valid entries for active workspace may be:
    Key: ActiveWorkspace:Location 
    Value: com.siemens.splm.client.search.SearchLocation
    
    Or
    
    Key: ActiveWorkspace:SubLocation 
    Value: com.siemens.splm.client.search:SavedSearchSubLocation
    
    
    Or
    
    Key: isRedLineMode
    Value: true or false
    """
    targetPage: str = ''
    businessObject: BusinessObject = None
    businessObjectType: str = ''
    styleSheetType: str = ''
    styleSheetLastModDate: datetime = None
    clientContext: StringMap5 = None


@dataclass
class GetStyleSheetOutput(TcBaseObj):
    """
    Output from getStyleSheet operation.
    
    :var processedPage: The name of the page that was processed. If the operation was invoked with processEntireXRT set
    to true, the this field is empty. If the operation was invoked with processEntireXRT set to false, this this field
    is populated with the page that was processed. Typically the value of the processedPage would be the same as the
    targetPage, except when the targetPage value is empty or not found in the stylesheet. In that case, then the first
    page in the stylesheet is processed.
    :var datasetInfo: The information about the stylesheet.
    :var objectSetMap: A Map (string/Teamcenter::Awp0XRTObjectSet) where key is the source field from the stylesheet
    and the value is the data to present inside that objectSet in the application.
    :var tablePropObjectMap: A map(string/vector of POM_object) where key is the name of the table property and the
    value is a vector of Table Row objects which represents the table row data for that property.
    :var localeMap: Map (string/string) where the key is the text, title, or titleKey string from the stylesheet and
    the value is the localized string.
    :var jtFileMap: Map (BusinessObject/string) where the key is the business object to render, and the value is the
    thumbnail ticket for the image to download.
    :var visiblePages: The visible pages in the XRT.  Each index in the array indicates whether that page can be
    displayed.
    :var classificationData: The classification data for the object to be rendered.
    :var objectToRender: The object to render. It could be the input object itself in the case of viewing a stylesheet,
    or the revise descriptor object in the case of the revise operation.
    :var patternMap: Map containing the patterns used for auto-assignable properties. The key is the property name, and
    the value is the patterns to use. Only auto assignable properties exist in the map.
    """
    processedPage: str = ''
    datasetInfo: GetStyleSheetDatasetInfo = None
    objectSetMap: ObjectSetMap4 = None
    tablePropObjectMap: TablePropObjectMap = None
    localeMap: StringMap5 = None
    jtFileMap: ThumbnailMap4 = None
    visiblePages: List[bool] = ()
    classificationData: List[GetStyleSheetClassificationData] = ()
    objectToRender: BusinessObject = None
    patternMap: PatternMap = None


@dataclass
class GetStyleSheetResponse(TcBaseObj):
    """
    Response sent to client from the getStyleSheet operation.
    
    :var output: The vector of output information. One for each input object.
    :var serviceData: The SOA service data.
    """
    output: List[GetStyleSheetOutput] = ()
    serviceData: ServiceData = None


@dataclass
class LoadDataForEditingInfo(TcBaseObj):
    """
    Structure represents the parameters required to save the edits on the input objects and submit them to a workflow
    process.
    
    :var obj: Input object
    :var propertyNames: Property names
    :var isPessimisticLock:  If true, the object needs to be checked out before making any edits (pessimistic locking).
    The locking of object is done during loadDataForEditing() call.If false, the operation takes optimistic lock on the
    object. In optimistic locking the object is locked (based on object last set date "lsd" property) before performing
    the actual save and lock is released after save is complete.
    """
    obj: BusinessObject = None
    propertyNames: List[str] = ()
    isPessimisticLock: bool = False


@dataclass
class LoadDataForEditingOutput(TcBaseObj):
    """
    Output structure of LoadDataForEditing operation
    
    :var obj: Object to start edit or its dependent objects
    :var objLsds: List of last save date of object or dependent objects.
    :var tableProperties: A map(string/list of POM_object) where key is the name of the table property and the value is
    a vector of Table Row objects which represents the table row data for that property.
    """
    obj: BusinessObject = None
    objLsds: List[ObjectLsdInfo] = ()
    tableProperties: TablePropObjectMap = None


@dataclass
class LoadDataForEditingResponse(TcBaseObj):
    """
    Response of LoadDataForEditing SOA
    
    :var outputs: A list of LoadDataForEditingOutput strutures which provide information about the editable status of
    the properties passed in the input. This information can be used by client to show which properties as editable.
    :var serviceData: Partial errors are returned in the Service Data. The following partial errors may be returned:
    
    302021: The input object is not modifiable.
    302022: The property is not modifiable.
    """
    outputs: List[LoadDataForEditingOutput] = ()
    serviceData: ServiceData = None


@dataclass
class PatternInfo(TcBaseObj):
    """
    Structure defining Pattern information.
    
    :var patterns: List of naming rule patterns.
    :var preferredPattern: The preferred pattern to use.
    :var condition: The condition value.
    """
    patterns: List[str] = ()
    preferredPattern: str = ''
    condition: str = ''


@dataclass
class PropertyNameValue(TcBaseObj):
    """
    Input Structure containing information about what properties are to be populated. 
    For normal (non-Table )Properties, user has to populate values. 
    For Table Properties, user has to fill the rowValues structure
    
    :var name: Property name string that needs to be modified.
    :var values: Property values string vector that will be used to update the input object with corresponding values.
    This vector contains string representation of the property value. The calling client is responsible for converting
    the different property types (int, float, date .etc) to a string using the appropriate toXXXString functions in the
    SOA client framework's Property class.
    :var rowValues: A list to hold child object structures. Child structure will be populated in case the property
    contains a collection of sub objects. E.g. table properties. For other properties, the list needs to be empty.
    """
    name: str = ''
    values: List[str] = ()
    rowValues: List[RowDataStruct] = ()


@dataclass
class RowDataStruct(TcBaseObj):
    """
    A structure to hold child information associated with the main object.
    It contains uid of the child, and its name value pairs for different properties.
    
    :var uid: UID for the child object. If the UID is blank  / NULL UID, a new object is created with details as
    populated in nameValues.
    :var nameValues: List of structure containing Information which needs to be saved. If UID is not blank then the
    properties populated in this structure will be updated. If UID is blank, then a new child object is created based
    on the name values input.
    """
    uid: str = ''
    nameValues: List[NameValueStruct] = ()


@dataclass
class SaveEditAndSubmitInfo(TcBaseObj):
    """
    Structure represents the parameters required to save the edits on the input objects and submit them to a workflow
    process.
    
    :var object: Object to save edit and submit to a workflow process.
    :var propertyNameValues: Property name and values structure that will contain all property names and corresponding
    values that needs to be saved.
    :var objLsds: The last set date information for the object.
    :var isPessimisticLock: The flag to control whether this method call performs optimistic locking or not.
    A false value means the operation takes optimistic lock on the object. In optimistic locking the object is locked
    (based on the object last saved date) before performing the actual save and lock is released after save is complete.
    A true value means the object needs to be checked out before making any edits (pessimistic locking). The locking of
    object is done during loadDataForEditing operation.
    :var workflowData: The workflow information that the input object needs to submit to a workflow. Workflow
    information is stored in a name and value string map (string/list of strings).  If workflowData map is empty,
    objects will not be submitted to any workflow.
    Supported keys:
    
    &bull;   submitToWorkflow: Boolean Property to define that input object need to submit to workflow process or not.
    It can contain true or false as value. 
    &bull;   processName: Process name string. 
    &bull;   processDescription: Process description string. 
    &bull;   processTemplate: Name of the process template to be used to create new workflow process. 
    &bull;   processAssignmentList: Name of the process assignment list to use while creating new workflow process. 
    
    If property submitToWorkflow value is true and processTemplate value is empty, then workflow process template to be
    used for workflow creation should be specified in the preference (<TypeName>_default_workflow_template) defined for
    the submitted object type.
    """
    object: BusinessObject = None
    propertyNameValues: List[PropertyNameValue] = ()
    objLsds: List[ObjectLsdInfo] = ()
    isPessimisticLock: bool = False
    workflowData: PropertyValues1 = None


@dataclass
class TableRow(TcBaseObj):
    """
    Structure containing pre-filled values of row information. The values can then be modified, and a subsequent SOA
    call to Save operation (saveEdits() SOA) will persist the row object in database.
    
    :var tableRowTypeName: The business object type name of the table row. This can be Fnd0TableRow or any subtype.
    :var tableRowData: A list of property values, where name is the column name (sub property name) of the table
    property. "dbValues" and "uiValues" are the list of database value and list of display values respectively.
    """
    tableRowTypeName: str = ''
    tableRowData: List[TableRowProperty] = ()


@dataclass
class TableRowProperty(TcBaseObj):
    """
    Structure containing pre-populated property values for columns of the row object.
    
    :var name: The name of the property. Caller can look up the property descriptor to get metadata about the property.
    :var dbValues: Proposed data base value for the property.
    :var uiValues: Proposed display values for the property.
    """
    name: str = ''
    dbValues: List[str] = ()
    uiValues: List[str] = ()


"""
Maps the object set source in the xml rendering style sheet to the business objects.
"""
ObjectSetMap4 = Dict[str, Awp0XRTObjectSet]


"""
Map of string to vector of strings. This map is used to return the auto assignable property name patterns to the client. The key is the property name and the value is the list of patterns.
"""
PatternMap = Dict[str, PatternInfo]


"""
Map (string, vector) that is a generic container that represents property values. The key is the property name and the value is the string representation of the property value.
"""
PropertyValues1 = Dict[str, List[str]]


"""
String map.
"""
StringMap5 = Dict[str, str]


"""
Maps the table property in the xml rendering style sheet to the Table Row objects.
"""
TablePropObjectMap = Dict[str, List[POM_object]]


"""
Maps a business object to a thumbnail file ticket.
"""
ThumbnailMap4 = Dict[BusinessObject, str]



from tcsoa.gen.Multisite._2019_06.services import SearchService as imp0
from tcsoa.gen.Multisite._2011_06.services import ImportExportService as imp1
from tcsoa.gen.Multisite._2019_06.services import ImportExportTCXMLService as imp2
from tcsoa.gen.Multisite._2014_10.services import ImportExportTCXMLService as imp3
from tcsoa.gen.Multisite._2007_06.services import ImportExportService as imp4
from tcsoa.base import TcService


class SearchService(TcService):
    publish = imp0.publish
    unpublish = imp0.unpublish


class ImportExportService(TcService):
    registerItemId = imp1.registerItemId
    remoteImport = imp4.remoteImport
    unregisterItemId = imp1.unregisterItemId


class ImportExportTCXMLService(TcService):
    remoteCheckin = imp2.remoteCheckin
    remoteCheckout = imp2.remoteCheckout
    remoteExport = imp3.remoteExport
    remoteImport = imp3.remoteImport
    remoteImportByUID = imp3.remoteImportByUID



from __future__ import annotations

from tcsoa.gen.Internal.ProjectManagement._2014_10.ScheduleManagement import PasteTaskContainer, DeferredSaveOption, DeferredSaveResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def pasteTasks(cls, inputs: List[PasteTaskContainer]) -> ServiceData:
        """
        The pasteTasks operation pastes the input ScheduleTask to the specified position in the Schedule. The operation
        takes as input the task being pasted, the new parent task under which the task has to be pasted and the
        position in the new parent task after which the task needs to be pasted. The operation performs the cut
        functionality is the input specifies that it's a cut operation. The task is removed from its orginal position
        in the Schedule if the input specifies so.
        """
        return cls.execute_soa_method(
            method_name='pasteTasks',
            library='Internal-ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def deferredSave(cls, deferredOption: DeferredSaveOption) -> DeferredSaveResponse:
        """
        This operation performs the following actions depending on the input specified 
        1.Start a deferred session on a Schedule. Starting a deferred session locks the Schedule so that the other
        users are not be able to update the Schedule in different sessions.
        2.Save the edits made on the Schedule in the deferred session to the database. 
        3.Exit the deferred session after saving the edits made on the Schedule in the deferred session to the
        database. On exit, the lock on the Schedule is released which enables the Schedule for editing by other users.
        4.Discards the contents of the deferred session.
        
        The following data is editable by the user who has started the deferred session on the Schedule when the
        Schedule is in deferred session
        1.Only scheduling attributes of the Schedule such as (planned start date, planned end date, work estimate,
        duration, constraint).
        2.Only scheduling attributes of the ScheduleTask such as (planned start date, planned end date, work estimate,
        duration, constraint).
        3.Create, delete and update of ScheduleTask, ResourceAssignment, TaskDependency and ScheduleMember.
        
        The following data cannot be edited by the user who has started the deferred session on the Schedule
        1.Execution data of ScheduleTask
        2.Insert Schedule, detatch Schedule actions.
        
        The following data can be edited on the Schedule by other users when the Schedule is in deferred session
        1.Only execution data of the Schedule and ScheduleTask. No other update is allowed on the Schedule and
        ScheduleTask.
        
        
        
        Exceptions:
        >ServiceException
        """
        return cls.execute_soa_method(
            method_name='deferredSave',
            library='Internal-ProjectManagement',
            service_date='2014_10',
            service_name='ScheduleManagement',
            params={'deferredOption': deferredOption},
            response_cls=DeferredSaveResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.DocumentManagement._2013_12.PrintOrRender import PrinterDefinitionResponse, RenderSubmitRequestInfo, PrintSubmitRequestInfo, SubmitRequestResponse
from tcsoa.base import TcService


class PrintOrRenderService(TcService):

    @classmethod
    def printSubmitRequest(cls, input: List[PrintSubmitRequestInfo]) -> SubmitRequestResponse:
        """
        This operation submits print requests for batch printing.
        
        Use cases:
        The client can call this operation to do batch printing. Batch printing lets you select workspace objects, such
        as Item, ItemRevision, or Dataset objects, and print the associated documents with system stamps and watermarks.
        """
        return cls.execute_soa_method(
            method_name='printSubmitRequest',
            library='DocumentManagement',
            service_date='2013_12',
            service_name='PrintOrRender',
            params={'input': input},
            response_cls=SubmitRequestResponse,
        )

    @classmethod
    def renderSubmitRequest(cls, input: List[RenderSubmitRequestInfo]) -> SubmitRequestResponse:
        """
        This operation submits render requests for rendering.
        
        Use cases:
        The client can call this operation to do the rendering on ItemRevision objects. When you render an ItemRevision
        object containing a dataset, you translate the associated file to an alternate format.
        """
        return cls.execute_soa_method(
            method_name='renderSubmitRequest',
            library='DocumentManagement',
            service_date='2013_12',
            service_name='PrintOrRender',
            params={'input': input},
            response_cls=SubmitRequestResponse,
        )

    @classmethod
    def getPrinterDefinitions(cls) -> PrinterDefinitionResponse:
        """
        This operation returns Print Definition information from the PrintConfiguration objects defined in Teamcenter.
        
        Use cases:
        The client wants to get the information required for the Batch Print operation printSubmitRequest.
        """
        return cls.execute_soa_method(
            method_name='getPrinterDefinitions',
            library='DocumentManagement',
            service_date='2013_12',
            service_name='PrintOrRender',
            params={},
            response_cls=PrinterDefinitionResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Fnd0ProxyTask, WorkspaceObject, TCCalendar, ScheduleTask, Schedule, ResourceAssignment, TaskDependency, TCCalendarEvent
from typing import List
from tcsoa.gen.ProjectManagement._2008_06.ScheduleManagement import ScheduleDeepCopyinfo, StringValContainer, TypedAttributeContainer
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class LoadScheduleContainer(TcBaseObj):
    """
    Contain the schedule uids and options for loading schedules
    
    :var scheduleUids: The UIDs of the schedules.
    :var schmgtOptions: The options for loading. Integer Option:
    "SM_Structure_Partial_Context"
    Values:
    0 =  load the full schedule including all sub schedules and their children .
    1 =   load only schedule summaries partially   
    
    Integer Option: 
    "SM_Structure_Load_Context"
    Values: 
    0 = loading schedule 
    1 = loading sub-schedule  
    4 = inserting sub schedule by reference
    Integer Option:
    "SM_Structure_Client_Context"
    Values:
    0 = RAC client
    1 = Server client (for Synchronous dispatcher)
    2 = MSP plugin client
    """
    scheduleUids: List[str] = ()
    schmgtOptions: SchMgtOptions = None


@dataclass
class LockRequest(TcBaseObj):
    """
    A request structure to lock the schedule.
    
    :var schedule: The schedule for which the lock request is
    :var requestLock: The request to lock (true) or unlock (false)
    """
    schedule: Schedule = None
    requestLock: bool = False


@dataclass
class LockResponse(TcBaseObj):
    """
    A Response for SOA "ManageScheduleLocks". This will indicate whether the lock request is successful or not.
    
    :var schedule: The schedule locked or unlocked
    :var lockState: The state of the lock (true=locked, false=not locked)
    :var requestSuccess: Was the request successful. True if the request was a success.
    """
    schedule: Schedule = None
    lockState: bool = False
    requestSuccess: bool = False


@dataclass
class LockResponses(TcBaseObj):
    """
    List of multiple "LockResponses".
    
    :var data: The service data
    :var responses: The individual responses
    """
    data: ServiceData = None
    responses: List[LockResponse] = ()


@dataclass
class MasterMetaData(TcBaseObj):
    """
    A container for the SubMasterMetaData.
    
    :var uid: The uid of a master schedule.
    :var start: The Start Date of that master schedule.
    :var finish: The Finish Date of that master schedule.
    """
    uid: str = ''
    start: datetime = None
    finish: datetime = None


@dataclass
class MultipleScheduleLoadResponses(TcBaseObj):
    """
    A container for load schedule responses
    
    :var scheduleData: Collection of the schedule load responses.
    :var serviceData: The ServiceData.
    """
    scheduleData: List[ScheduleLoadResponse] = ()
    serviceData: ServiceData = None


@dataclass
class ProxyTaskContainer(TcBaseObj):
    """
    Container for Proxy Task
    
    :var schedule: The schedule were the proxy is being created
    :var sublevels: Number of sublevels (if this should be mirrored in the sub schedules (-1 =  all, 0 = only this
    schedule, 1= first level, 2 = 1st and 2nd level, etc)
    :var taskTag: The real task being proxied.
    :var refTag: The reference task in the schedule
    """
    schedule: Schedule = None
    sublevels: int = 0
    taskTag: ScheduleTask = None
    refTag: WorkspaceObject = None


@dataclass
class ProxyTaskResponse(TcBaseObj):
    """
    Response for createProxyTasks call.
    
    :var schedule: The schedule of the proxy task.
    :var proxyTask: The proxy task
    :var subScheduleAdditions: A collection containing a ProxyTaskResponse for each sub schedules addition
    """
    schedule: Schedule = None
    proxyTask: Fnd0ProxyTask = None
    subScheduleAdditions: List[ProxyTaskResponse] = ()


@dataclass
class ProxyTaskResponses(TcBaseObj):
    """
    A collection of ProxyTaskResponses returned for createProxyTasks call.
    
    :var data: The service data
    :var responses: A collection of ProxyTaskResponse
    """
    data: ServiceData = None
    responses: List[ProxyTaskResponse] = ()


@dataclass
class RefreshScheduleContainer(TcBaseObj):
    """
    Container for refreshing schedule
    
    :var masterScheduleUid: The UID of the master schedule.
    :var componentUid: The UID of the object to refresh
    :var refreshOptions: The options for refresh. Integer Option:
    "SM_Structure_Partial_Context"
    Values:
    0 =  load the full schedule including all sub schedules and their children .
    1 =   load only schedule summaries partially   
    
    Integer Option: 
    "SM_Structure_Load_Context"
    Values: 
    6 = refresh the schedule
    7 = load all objects currently not in the client; compliment
    
    Integer Option:
    "SM_Structure_Client_Context"
    Values:
    0 = RAC client
    1 = Server client (for Synchronous dispatcher)
    2 = MSP plugin client
    :var clientCache: List of currently loaded components for the object being refreshed.
    :var lastModifiedDate: The last modified date for the component represented by componentUID.
    """
    masterScheduleUid: str = ''
    componentUid: str = ''
    refreshOptions: SchMgtOptions = None
    clientCache: SchmgtClientCache = None
    lastModifiedDate: datetime = None


@dataclass
class SchMgtIntegerOption(TcBaseObj):
    """
    Schedule Manager Integer Option
    
    :var name: The name of the option
    :var value: The value of the option
    """
    name: str = ''
    value: int = 0


@dataclass
class SchMgtIntegerOptionAsync(TcBaseObj):
    """
    SchMgtIntegerOptions for Async operation
    
    :var name: The name of the option
    :var value: The value of the option
    """
    name: str = ''
    value: int = 0


@dataclass
class SchMgtLogicalOption(TcBaseObj):
    """
    Schedule Manager Logical Option
    
    :var name: The name of the option
    :var value: The value of the option (True or False)
    """
    name: str = ''
    value: bool = False


@dataclass
class SchMgtLogicalOptionAsync(TcBaseObj):
    """
    SchMgtLogicalOptions for Async operation
    
    :var name: The name of the option
    :var value: The value of the option (True or False)
    """
    name: str = ''
    value: bool = False


@dataclass
class SchMgtOptions(TcBaseObj):
    """
    Schedule Management Options
    
    :var logicalOptions: Logical options
    :var integerOptions: Integer Options
    :var stringOptions: String Options
    """
    logicalOptions: List[SchMgtLogicalOption] = ()
    integerOptions: List[SchMgtIntegerOption] = ()
    stringOptions: List[SchMgtStringOption] = ()


@dataclass
class SchMgtOptionsAsync(TcBaseObj):
    """
    SchMgtOptions for Async operation
    
    :var logicalOptions: Logical options
    :var integerOptions: Integer Options
    :var stringOptions: String Options
    """
    logicalOptions: List[SchMgtLogicalOptionAsync] = ()
    integerOptions: List[SchMgtIntegerOptionAsync] = ()
    stringOptions: List[SchMgtStringOptionAsync] = ()


@dataclass
class SchMgtStringOption(TcBaseObj):
    """
    Schedule Manager String Option
    
    :var name: The name of the option
    :var value: The value of the option.
    """
    name: str = ''
    value: str = ''


@dataclass
class SchMgtStringOptionAsync(TcBaseObj):
    """
    SchMgtStringOptions for Async operation
    
    :var name: The name of the option
    :var value: The value of the option.
    """
    name: str = ''
    value: str = ''


@dataclass
class ScheduleCopyOptionsContainer(TcBaseObj):
    """
    Container for schedule copy options
    
    :var name: The name of the new schedule
    :var description: The description of the new schedule. Valid value -can be empty string.
    :var id: The ID of the new schedule.
    :var revId: The revId of the new schedule.
    :var scheduleToCopy: The tag to the schedule to copy.
    :var options: Copy schedule options 1) logical options  -{bool copyBaselines,bool loadOnResponse,bool
    resetWork,bool copyProxyTasks,bool  copycrossScheduleDependencies } 2) integerOptions -{ int ideepCopyCount } 3)
    stringOptions -{bool deepcopyrequired }
    :var stringValueContainer: Additional attributes for the new schedule.(optional)
    :var typedAttributesContainer: Additional  attributes for the new schedule.(optional)
    :var copyInfo: Additional metadata for the copy.(optional)
    """
    name: str = ''
    description: str = ''
    id: str = ''
    revId: str = ''
    scheduleToCopy: Schedule = None
    options: SchMgtOptions = None
    stringValueContainer: List[StringValContainer] = ()
    typedAttributesContainer: List[TypedAttributeContainer] = ()
    copyInfo: List[ScheduleDeepCopyinfo] = ()


@dataclass
class ScheduleCopyOptionsContainerAsync(TcBaseObj):
    """
    ScheduleCopyOptionsContainer for copySchedule
    
    :var name: The name of the new schedule
    :var description: The description of the new schedule. Valid value -can be empty string.
    :var id: The ID of the new schedule.
    :var revId: The revId of the new schedule.
    :var scheduleToCopy: The tag to the schedule to copy.
    :var options: Copy schedule options 1) logical options  -{bool copyBaselines,bool loadOnResponse,bool
    resetWork,bool copyProxyTasks,bool  copycrossScheduleDependencies } 2) integerOptions -{ int ideepCopyCount } 3)
    stringOptions -{bool deepcopyrequired }
    :var stringValueContainer: Additional  attributes for the new schedule.(optional)
    :var typedAttributesContainer: Additional  attributes for the new schedule.(optional)
    :var copyInfo: This parameter specifies any additional metadata for the copy of the schedule. This
    'ScheduleDeepCopyinfoAsync' data structure holds the relevant information regarding applicable deep copy rules.
    This is optional parameter.
    """
    name: str = ''
    description: str = ''
    id: str = ''
    revId: str = ''
    scheduleToCopy: Schedule = None
    options: SchMgtOptionsAsync = None
    stringValueContainer: List[StringValContainerAsync] = ()
    typedAttributesContainer: List[TypedAttributeContainerAsync] = ()
    copyInfo: List[ScheduleDeepCopyinfoAsync] = ()


@dataclass
class ScheduleDeepCopyinfoAsync(TcBaseObj):
    """
    ScheduleDeepCopyinfoAsync container
    
    :var objectComp: A tag representing object on which the
    deep copy action need to be performed.
    :var relation: A string representing the name the relation  that need to be deep copied. Valid value can be
    IMANRelation.
    :var objName: A string representing the new name for the new copy.
    of the object represented by otherSideObjectTag. The value for
    the newName will be null if the 'action' is not CopyAsObject or
    ReviseAndRelateToLatest.
    :var operationType: An integer representing the action to be performed on the object represented by
    'otherSideObjectTag'.
    The values for action are:
    CopyAsObjectType = 0,
    CopyAsRefType = 1,
    DontCopyType =2,
    RelateToLatest = 3,
    ReviseAndRelateToLatest = 4
    :var isTargetPrimary: A Boolean representing whether the given item
    revision is a primary object in the relation that need
    to be deep copied.
    :var isRequired: A Boolean representing whether the deep information is from a mandatory deep copy rule configured
    by the administrator or not.
    """
    objectComp: BusinessObject = None
    relation: str = ''
    objName: str = ''
    operationType: int = 0
    isTargetPrimary: bool = False
    isRequired: bool = False


@dataclass
class ScheduleLoadResponse(TcBaseObj):
    """
    Container for the load schedules response.
    
    :var schedule: The schedule
    :var scheduleTasks: All the tasks in this schedule.
    :var resourceAssignments: All the resource assignments in this schedule.
    :var taskDependencies: All the task dependencies in this schedule.
    :var calendars: All the calendars referenced in this schedule.
    :var calendarEvents: All the calendar events referenced in the calendars.
    :var submasterdata: The metadata for master-schedules of this schedule and all sub-schedules.
    :var namedStringLists: Currently used by refresh to return deleted objects to the client.
    """
    schedule: Schedule = None
    scheduleTasks: List[ScheduleTask] = ()
    resourceAssignments: List[ResourceAssignment] = ()
    taskDependencies: List[TaskDependency] = ()
    calendars: List[TCCalendar] = ()
    calendarEvents: List[TCCalendarEvent] = ()
    submasterdata: List[SubMasterMetaData] = ()
    namedStringLists: List[SchmgtNamedStringList] = ()


@dataclass
class SchmgtClientCache(TcBaseObj):
    """
    The list of names lists.
    
    :var namedStringLists: Array of named string lists of components uid
    """
    namedStringLists: List[SchmgtNamedStringList] = ()


@dataclass
class SchmgtNamedStringList(TcBaseObj):
    """
    A vector of lists.
    
    :var name: A unique identifier for this list (possibly UID).
    :var list: The list of  UIDs contained by this object.
    :var aggregate: Not Used
    """
    name: str = ''
    list: List[str] = ()
    aggregate: float = 0.0


@dataclass
class StringValContainerAsync(TcBaseObj):
    """
    string value container for Async operation
    
    :var key: A string key identifying the attribute.
    :var value: A string representation of the value of the attribute.
    :var type: An integer to help determine the data type of the attribute. Valid values- {
    1=String_type,2=Integer_type,3=Long_type,4=Double_type,5=Float_type,6=Boolean_type,7=Date_type,8=Cal_type }
    """
    key: str = ''
    value: str = ''
    type: int = 0


@dataclass
class SubMasterMetaData(TcBaseObj):
    """
    A container for the sub-schedule master MetaData.
    
    :var subschedule: The tag of the sub schedule.
    :var masterdata: Collection of metadata for all master schedules of this schedule and it's sub-schedules.
    """
    subschedule: Schedule = None
    masterdata: List[MasterMetaData] = ()


@dataclass
class TaskExecUpdate(TcBaseObj):
    """
    The structure which would be used to send back and forth the data for execution view for tasks.
    
    :var task: The task being updated.
    :var updateAS: Must be true when updating the Actual Start Date.
    :var newAS: The new Actual Start Date to set (null is allowed). updateAS must also be to true to update this value.
    :var updateAF: Must be true when updating the Actual Finish Date.
    :var newAF: The new Actual Finish Date to set (null is allowed). updateAF must also be to true to update this value.
    :var newPC: The new percent complete for the task (0-100) (-1 for no  update)
    :var newStatus: The new status for the task. Valid values are the status strings listed in the status LOV. ("" or
    null for no update)
    :var newWC: The new work complete to set (-1 means no update)
    :var newWR: The new work remaining to set (-1 means no update)
    """
    task: ScheduleTask = None
    updateAS: bool = False
    newAS: datetime = None
    updateAF: bool = False
    newAF: datetime = None
    newPC: float = 0.0
    newStatus: str = ''
    newWC: int = 0
    newWR: int = 0


@dataclass
class TypedAttributeContainerAsync(TcBaseObj):
    """
    TypedAttributeContainerAsync conatiner for Async operation
    
    :var type: The object type
    :var attributes: A collection of updates
    """
    type: str = ''
    attributes: List[StringValContainerAsync] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.ProjectManagement._2011_06.ScheduleManagement import ScheduleCopyOptionsContainer, LockRequest, ProxyTaskContainer, LoadScheduleContainer, ScheduleCopyOptionsContainerAsync, RefreshScheduleContainer, ProxyTaskResponses, TaskExecUpdate, LockResponses, MultipleScheduleLoadResponses
from tcsoa.gen.BusinessObjects import Schedule
from tcsoa.gen.ProjectManagement._2008_06.ScheduleManagement import ScheduleCopyResponses
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def loadSchedules(cls, loadScheduleContainers: List[LoadScheduleContainer]) -> MultipleScheduleLoadResponses:
        """
        The operation loads into memory all the objects belonging to a schedule. The objects are those needed to manage
        and validate the schedule. The objects are can be  instances of Schedule, ScheduleTask, ResourceAssignment,
        TaskDependency, TCCalendar TCCalendarEvents and 'SubMasterMetaData' .
        This operation requires a load option which defines how it behaves, such as loading sections of a structured
        schedule on demand or loading the full schedule if system resources are available to accomplish request. The
        load option is defined by the preference "SM_Structure_Partial_Context" which can be set to one of the
        following values: 
        1)    0 -> load all the schedule objects
        2)    1 -> load the schedule objects on demand.
        The default load option is 0 (load all objects).
        """
        return cls.execute_soa_method(
            method_name='loadSchedules',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'loadScheduleContainers': loadScheduleContainers},
            response_cls=MultipleScheduleLoadResponses,
        )

    @classmethod
    def manageScheduleLocks(cls, requests: List[LockRequest]) -> LockResponses:
        """
        This is an operation that is used to manage concurrent access to schedule data. The server concurrent access
        capability allows schedule data to be accessed simultaneously from different sessions. To ensure data
        integrity, the server enforces a first-in-first-out execution logic by locking out access to the same data from
        other sessions until the lock is removed. During the lock period the session that owns the lock is allowed to
        add and update the schedule data. For deferred save or bulk update, the operation is used to purposely lock the
        schedule data until an unlock request is received.
        """
        return cls.execute_soa_method(
            method_name='manageScheduleLocks',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'requests': requests},
            response_cls=LockResponses,
        )

    @classmethod
    def refreshScheduleObject(cls, refreshScheduleContainer: RefreshScheduleContainer) -> MultipleScheduleLoadResponses:
        """
        The server allows concurrent access to a schedule data and because of this capability updates and additions can
        be added to the schedule data from other sessions apart from a client session. This operation loads all the
        objects in a schedule since the last modified date in the client session. The client can also specify the date
        from which to begin the load. The operation would then return all schedule objects that were modified from the
        specified date. It may also include objects modified in the client's session. The objects are Schedule,
        ScheduleTask, ResourceAssignment, TaskDependency, TCCalendar TCCalendarEvents and SubMasterMetaData.
        """
        return cls.execute_soa_method(
            method_name='refreshScheduleObject',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'refreshScheduleContainer': refreshScheduleContainer},
            response_cls=MultipleScheduleLoadResponses,
        )

    @classmethod
    def copySchedules(cls, containers: List[ScheduleCopyOptionsContainer]) -> ScheduleCopyResponses:
        """
        This operation makes a deep copy of the schedule with options to reset work and copy existing baselines. 
        The following  are the options:  'resetWork' (false if not provided), 
        'copyBaselines' (false if not provided), 
        'copyProxyTasks' (false if not provided), 
        'copyCrossScheduleDependencies' (false if not provided ) 
        
        The information needed to copy the schedule is specified in the 'ScheduleCopyOptionsContainer' structure. It
        returns 'ScheduleCopyResponses' which will have information of copied  schedules and 'ServiceData'. Errors will
        be returned in the list of partial errors in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='copySchedules',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'containers': containers},
            response_cls=ScheduleCopyResponses,
        )

    @classmethod
    def copySchedulesAsync(cls, schToCopy: List[ScheduleCopyOptionsContainerAsync]) -> None:
        """
        This operation copies schedule asynchronously. This is a server function for copySchedule asynchronous
        operation. When this operation is called it files asynchronous request to copy schedule and hence it will not
        block the client for whole copy schedule operation. It will release the client after filing asynchronous
        request immediately so user can perform other operation. The information required to copy the schedule
        asynchronously is passed as parameter to the operation as ScheduleCopyOptionsContainerAsync data structure
        """
        return cls.execute_soa_method(
            method_name='copySchedulesAsync',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'schToCopy': schToCopy},
            response_cls=None,
        )

    @classmethod
    def copySchedulesAsyncClient(cls, schToCopy: List[ScheduleCopyOptionsContainerAsync]) -> bool:
        """
        This operation copies schedule asynchronously. The information required to copy the schedule asynchronously is
        passed as parameter to the operation as 'ScheduleCopyOptionsContainerAsync' data structure.
        """
        return cls.execute_soa_method(
            method_name='copySchedulesAsyncClient',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'schToCopy': schToCopy},
            response_cls=bool,
        )

    @classmethod
    def updateTaskExecution(cls, taskUpdateExec: List[TaskExecUpdate]) -> ServiceData:
        """
        Updates the execution data attributes of Schedule Tasks. The updates are specified through 'TaskExecUpdate' 
        structures. actual start date, actual finish date, percent complete, status, work complete and work remaining
        attributes on schedule task can be updated through this operation.
        """
        return cls.execute_soa_method(
            method_name='updateTaskExecution',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'taskUpdateExec': taskUpdateExec},
            response_cls=ServiceData,
        )

    @classmethod
    def createProxyTasks(cls, newProxyTasks: List[ProxyTaskContainer]) -> ProxyTaskResponses:
        """
        This operation creates proxy tasks. The information needed to create proxy tasks are specified in the
        'ProxyTaskContainer' structure. It returns 'ProxyTaskResponses' which will have information of created proxy
        tasks and 'ServiceData'. Errors will be returned in the list of partial errors in the 'ServiceData' if
        operation fails to create proxy task.
        """
        return cls.execute_soa_method(
            method_name='createProxyTasks',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'newProxyTasks': newProxyTasks},
            response_cls=ProxyTaskResponses,
        )

    @classmethod
    def deleteScheduleAsync(cls, schToDelete: Schedule) -> None:
        """
        Deletes a specified schedule asynchronously. The operation returns after triggering the asynchronous delete on
        the schedule.
        """
        return cls.execute_soa_method(
            method_name='deleteScheduleAsync',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'schToDelete': schToDelete},
            response_cls=None,
        )

    @classmethod
    def deleteScheduleAsyncClient(cls, schToDelete: Schedule) -> bool:
        """
        Deletes the specified schedule asynchronously but will check that there is a connection between client and
        server before performing the delete action. After the check is performed successfully a call to
        'deleteScheduleAsync' operation is made to finalize the deletion of the schedule. See operation
        'deleteScheduleAsync' for more details. 
        """
        return cls.execute_soa_method(
            method_name='deleteScheduleAsyncClient',
            library='ProjectManagement',
            service_date='2011_06',
            service_name='ScheduleManagement',
            params={'schToDelete': schToDelete},
            response_cls=bool,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ScheduleTask, Discipline, POM_object
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AssignmentCreateContainer(TcBaseObj):
    """
    The information needed to create a new assignment to a task.
    
    :var task: The task to assign the resource.
    :var resource: The resource (User, Group, Role, Resource Pool) being assigned (or null if discipline assignment).
    :var discipline: The discipline of the assignee (if User) or the discipline for the assignment.
    :var assignedPercent: The percentage effort being assigned (I.E. 50.0 == 50%)
    """
    task: ScheduleTask = None
    resource: POM_object = None
    discipline: Discipline = None
    assignedPercent: float = 0.0



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindValueInput(TcBaseObj):
    """
    Structure containing the additional criteria value(s) and unit systems which should be used to find the values for
    a certain attribute.
    
    :var attributeID: The attribute ID to find the values for.
    :var classID: The unique ID of the classification class in which to search (can be empty to find all values in all
    classes).
    :var searchInBothUnitSystems: Indicates to search for values stored in both unit systems. True to find the values
    stored in both the unit system, false otherwise.
    :var activeUnitsystem: The current unit system, 0 for metric, 1 for non-metric.
    :var attributeValues: A vector of AttributeValue structures containing attribute Ids and their corresponding UI
    values to use additionally to find the values for 'attributeID'.
    """
    attributeID: int = 0
    classID: str = ''
    searchInBothUnitSystems: bool = False
    activeUnitsystem: int = 0
    attributeValues: List[AttributeValue] = ()


@dataclass
class FindValuesOutput(TcBaseObj):
    """
    Structure containing the value(s) which have been found for the attribute.
    
    :var foundValues: The found values.
    """
    foundValues: List[FoundValue] = ()


@dataclass
class FindValuesResponse(TcBaseObj):
    """
    Structure containing the vector of found values. Any failures with the conversion will be mapped to the error
    message in the ServiceData list of partial errors.
    
    :var findValuesOutputs: The found values.
    :var svcData: Any failures with the operation will be mapped to the error message in the ServiceData list of
    partial errors.
    """
    findValuesOutputs: List[FindValuesOutput] = ()
    svcData: ServiceData = None


@dataclass
class FoundValue(TcBaseObj):
    """
    Structure containing the existing values, the count how many times the value is used and in which unit system.
    
    :var unitSystem: The unit system in which the value was found. 0 for metric, 1 for non-metric.
    :var value: The found value.
    :var count: The count how many times the value was found.
    """
    unitSystem: int = 0
    value: str = ''
    count: int = 0


@dataclass
class AttributeValue(TcBaseObj):
    """
    This structure contains the attribute ID and the corresponding value.
    
    :var id: The ID of the attribute.
    :var value: The value of the attribute.
    The calling client is responsible for converting the different property types (int, float, date etc) to a string
    using the appropriate functions in the SOA client framework's Property class.
    """
    id: int = 0
    value: str = ''



from __future__ import annotations

from tcsoa.gen.Workflow._2010_09.Workflow import ApplyTemplateInput, ApplyTemplateResponse
from typing import List
from tcsoa.base import TcService


class WorkflowService(TcService):

    @classmethod
    def applyTemplateToProcesses(cls, applyTemplateInput: List[ApplyTemplateInput], processingMode: int) -> ApplyTemplateResponse:
        """
        Apply the specified templates to all active workflow processes that are based on earlier versions of the
        templates
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='applyTemplateToProcesses',
            library='Workflow',
            service_date='2010_09',
            service_name='Workflow',
            params={'applyTemplateInput': applyTemplateInput, 'processingMode': processingMode},
            response_cls=ApplyTemplateResponse,
        )

    @classmethod
    def applyTemplateToProcessesAsync(cls, applyTemplateInput: List[ApplyTemplateInput]) -> None:
        """
        Apply the specified templates to all active workflow processes that are based on earlier versions of the
        template.
        """
        return cls.execute_soa_method(
            method_name='applyTemplateToProcessesAsync',
            library='Workflow',
            service_date='2010_09',
            service_name='Workflow',
            params={'applyTemplateInput': applyTemplateInput},
            response_cls=None,
        )



from tcsoa.gen.ProjectManagementAw._2019_06.services import ScheduleManagementAwService as imp0
from tcsoa.gen.ProjectManagementAw._2017_06.services import ScheduleManagementAwService as imp1
from tcsoa.gen.ProjectManagementAw._2018_12.services import ScheduleManagementAwService as imp2
from tcsoa.gen.ProjectManagementAw._2015_07.services import ScheduleManagementAwService as imp3
from tcsoa.gen.ProjectManagementAw._2020_05.services import ScheduleManagementAwService as imp4
from tcsoa.gen.ProjectManagementAw._2016_12.services import ScheduleManagementAwService as imp5
from tcsoa.base import TcService


class ScheduleManagementAwService(TcService):
    claimAssignments = imp0.claimAssignments
    createMasterSchedule = imp1.createMasterSchedule
    createMasterScheduleAsync = imp1.createMasterScheduleAsync
    createMultipleTaskDeliverables = imp2.createMultipleTaskDeliverables
    createTaskDeliverables = imp3.createTaskDeliverables
    designateDiscipline = imp2.designateDiscipline
    designateDisciplineAsync = imp2.designateDisciplineAsync
    getResourceChartInfo = imp4.getResourceChartInfo
    loadAssignedTasks = imp0.loadAssignedTasks
    loadBaseline = imp2.loadBaseline
    loadSchedule = imp5.loadSchedule
    loadSchedule2 = imp2.loadSchedule2
    recalculateSchedules = imp2.recalculateSchedules
    recalculateSchedulesAsync = imp2.recalculateSchedulesAsync
    removeAssignments = imp5.removeAssignments
    removeAssignmentsAsync = imp5.removeAssignmentsAsync
    replaceMemberAssignments = imp5.replaceMemberAssignments
    replaceMemberAssignmentsAsync = imp5.replaceMemberAssignmentsAsync
    shiftSchedules = imp4.shiftSchedules
    shiftSchedulesAsync = imp4.shiftSchedulesAsync



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanQuery
from typing import List
from tcsoa.gen.Internal.Query._2012_02.SavedQuery import SavedQuerySortAttribute
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SavedQueryClause3(TcBaseObj):
    """
    Holds the definition of a single saved query clause.  A query clause represents a single attribute that the end
    user can supply as search criteria.  Each saved query is made up of any number of ordered clauses.  Each attribute
    in this list is displayed to the end user allowing them to enter a search value.  Information about an attribute is
    attached with LOV or not also specified.  More common information includes the L10N key to be used when converting
    the attribute name for display to the user.  Also included is the logical operation to be used when evaluating this
    attribute in the saved query, and the comparison to be made between the user entered value and the value in the
    database.
    
    :var attributeName: Name of the attribute.
    :var entryL10NKey: The L10N key used when displaying this attribute to the user.
    :var entryNameDisplay: The translated version of the L10N key that will be displayed to the user.
    :var logicalOperator: Logical operator used when evaluating the clause.  This can be "AND", "OR" or "".
    :var mathOperator: Math operation to use when evaluating the clause.  This can be "=", "!=", ">", ">=", "<", "<=",
    "IS_NULL" or "IS_NOT_NULL".
    :var defaultInternalValue: Default internal value used for this attribute.
    :var defaultDisplayValue: Default display value used for this attribute.
    :var hasLOV: Indicates whether a LOV is attached to this attribute or not.
    :var keyLovId: Id of the key LOV for this attribute.
    :var attributeType: Type of attribute as defined in the AttributeDescriptor definition. Currently used to determine
    the correct attribute type to decide how the particular clause should be handled in the search form.
    """
    attributeName: str = ''
    entryL10NKey: str = ''
    entryNameDisplay: str = ''
    logicalOperator: str = ''
    mathOperator: str = ''
    defaultInternalValue: str = ''
    defaultDisplayValue: str = ''
    hasLOV: bool = False
    keyLovId: int = 0
    attributeType: int = 0


@dataclass
class SavedQueryDefinition3(TcBaseObj):
    """
    Contains the clauses and sort attributes that make up a saved query definition.  Each of the search clauses and
    sort attributes in the definition are order dependent.  The order of the clause attributes indicates how the 'AND'
    and 'OR' logic of the saved query will be evaluated.  The order of the sort attributes indicates the precedence of
    the results sorting.
    
    :var query: ImanQuery definition on the server.
    :var clauses: A list of ordered set of clauses that make up the query definition.  Each clause represents an
    attribute where the user can enter a value.  Each of these values is used to make up the overall query.
    :var sortAttributes: Optional list of attributes to be used for default sorting.  The ascending or descending order
    is included in the definition.  The order that the attributes appear in the vector indicate the order that they
    will impact the sorting.
    """
    query: ImanQuery = None
    clauses: List[SavedQueryClause3] = ()
    sortAttributes: List[SavedQuerySortAttribute] = ()


@dataclass
class DescribeSavedQueryDefnResponse3(TcBaseObj):
    """
    Holds the detailed definition data for the requested saved queries.  Each saved query definition will include
    search clauses and default sort attributes.  Each search clause attribute will be shown to the user during a query,
    and will allow the user to enter a value.  Each clause can have a default value and operators indicating how the
    value is evaluated.  The sort attributes allow the saved query to define the default sorting to use when this query
    is executed.  This can be altered by the end user from the sort dialog.
    
    :var definitions: A list of Saved query definitions data.  One definition is returned for each ImanQuery passed as
    input.  Each definition includes all the search criteria and sort attributes defined for each saved query.
    :var serviceData: Returned service data
    """
    definitions: List[SavedQueryDefinition3] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class LicenseStatus(TcBaseObj):
    """
    This structure  holds the number of licenses purchased and the number of licenses used.
    
    :var numPurchasedLicenses: The number of licenses purchased.
    :var numUsedLicenses: The number of licenses used.
    """
    numPurchasedLicenses: int = 0
    numUsedLicenses: int = 0


@dataclass
class LicenseStatusResponse(TcBaseObj):
    """
    This structure  holds a list of license statuses for the each given user license type.
    
    :var licStatus: List of LicenseStatus objects, one for each ActivateUserInput object.
    :var serviceData: Object with all activated User objects in updated list and any errors that occurred during
    activating users.
    """
    licStatus: List[LicenseStatus] = ()
    serviceData: ServiceData = None


@dataclass
class ActivateUserInput(TcBaseObj):
    """
    This structure holds the User object to be activated and the license level to be granted for the user.
    
    :var licenseLevel: The license level to be granted to the user.
    :var targetUser: The user to be activated and granted the licnese level.
    """
    licenseLevel: int = 0
    targetUser: BusinessObject = None


@dataclass
class DeactivateUserInput(TcBaseObj):
    """
    This structure holds the target user to be deactivated and new user and group to take the ownership of objects
    owned by the target user.
    
    :var targetUser: The user to be deactivated.
    :var newUser: The new user to take ownership transferred from the deactivated user.
    :var newGroup: The new group to take ownership transferred from the deactivated user.
    """
    targetUser: BusinessObject = None
    newUser: BusinessObject = None
    newGroup: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RuntimeBusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindNodeInContextInputInfo(TcBaseObj):
    """
    Input struct for the find node in context service
    
    :var clientID: Client ID
    :var context: The topline that defines the search scope.
    :var nodes: The nodes to search.
    :var byIdOnly: If true all abs occs with the same Id will be search for, if no exact apn is matched.
    :var allContexts: If true then all contexts will be searched otherwise only the current context will be searched if
    no current context specified at the time then the context of the topline is used.
    :var inContextLine: A more specific scope to searh in.
    :var relationTypes: A list of relation types, currently only FND_TraceLink relation is supported.
    :var relationDirection: The relation direction to search. The value includes 1(primary), 2(secondary) and 0(primary
    and secondary). It is only valid if the relation types are given.
    :var relationDepth: The depth to search, -1 to search all levels and any other positive integer value to search up
    to that level. It is only valid if the relation types are given.
    """
    clientID: str = ''
    context: BusinessObject = None
    nodes: List[BusinessObject] = ()
    byIdOnly: bool = False
    allContexts: bool = False
    inContextLine: BusinessObject = None
    relationTypes: List[str] = ()
    relationDirection: int = 0
    relationDepth: int = 0


@dataclass
class MatchObjectsAgainstVariantRulesArg(TcBaseObj):
    """
    This structure provides input parameters for the matchObjectsAgainstVariantRules operation.
    
    :var objects: The objects whose variant definition is to be checked against the set of variant rules or product
    variants. Currently only BOMLines are accepted.
    :var variantRules: The variant rules or product variants which are to be applied consecutively to each object. The
    objects must be of type 'VariantRule' or 'Mfg0BvrProductVariant', respectively.
    """
    objects: List[RuntimeBusinessObject] = ()
    variantRules: List[BusinessObject] = ()


@dataclass
class MatchObjectsAgainstVariantRulesResponse(TcBaseObj):
    """
    The response structure for the matchObjectsAgainstVariantRules operation.
    
    :var serviceData: The ServiceData object for this request.
    :var results: A list of result structures; each entry corresponds to an entry in the
    MatchObjectsAgainstVariantRulesArg vector passed to the matchObjectsAgainstVariantRules operation.
    """
    serviceData: ServiceData = None
    results: List[MatchObjectsAgainstVariantRulesResult] = ()


@dataclass
class MatchObjectsAgainstVariantRulesResult(TcBaseObj):
    """
    A set of data collected by the matchObjectsAgainstVariantRules operation for a specific
    MatchObjectsAgainstVariantRulesArg structure.
    
    :var matrix: A map that holds for each of the runtime objects supplied in the MatchObjectsAgainstVariantRulesArg
    structure the matching variant rules or product variants.
    :var warnings: A list of localized warning messages that describe inconsistencies in the configuration of the
    involved windows, if the variant configuration of a window does not satisfy the configuration of a supplied variant
    rule or product variant. The warnings are of the form "The window configuration for structure "<top line title>""
    is missing the following variant rule/product variant(s): <rule-name>, ...".
    """
    matrix: VariantRuleMap = None
    warnings: List[str] = ()


"""
A map that collects for a set of runtime objects a list of variant rules or, alternatively, product variants. This type maps a 'RuntimeBusinessObject' to a list of 'BusinessObject's.
"""
VariantRuleMap = Dict[RuntimeBusinessObject, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.Reports._2015_10.CrfReports import GetOfficeStyleSheetResponse
from tcsoa.gen.BusinessObjects import ReportDefinition
from tcsoa.base import TcService


class CrfReportsService(TcService):

    @classmethod
    def getOfficeStylesheets(cls, reportDefinition: ReportDefinition) -> GetOfficeStyleSheetResponse:
        """
        The operation returns the list of associated Office stylesheets
        """
        return cls.execute_soa_method(
            method_name='getOfficeStylesheets',
            library='Reports',
            service_date='2015_10',
            service_name='CrfReports',
            params={'reportDefinition': reportDefinition},
            response_cls=GetOfficeStyleSheetResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Visualization._2017_05.DataManagement import Snapshot3DInfoResponse2, Snapshot3DInfoInput2
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def getSnapshot3DInfo2(cls, snapshot3DInputList: List[Snapshot3DInfoInput2]) -> Snapshot3DInfoResponse2:
        """
        This service is used by the visualization integrations to gather the information needed to open a Product View
        (i.e. a SnapShotVisData Dataset) and apply it to recreate a 3D scene. The service gets SnapShotViewData Dataset
        information for a list of Product Views  of interest including all the named reference file information, all
        the supporting objects related to the Dataset by GRM or named reference, and all the visible lines referenced.
        This information is used by the viewer to open up the appropriate structure and configuration, expand the
        structure and load the visible lines , and recreate the 3D scene by applying the viewFile PLMXML.
        
        The client specifies  the SnapShotViewData Dataset objects to be opened, and the objects are added to the
        Snapshot3DInfoInput2 input structure. One or more SnapShotViewData Dataset objects can be added to the list to
        get information on many Product Views in batch. If more than one object is used, it is important to use unique
        clientId strings in the input, since they are used as keys in the return output map. If the same clientId
        string is used, information will be lost.
        
        The related objects returned are used to identify the top line of the structure (e.g. VisTopLevelRef), and to
        get the configuration of the structure to open (e.g. VisStructureContext object).
        
        The visible lines are used to expand the structure and load the appropriate occurrences in order to recreate
        the 3D scene. This consists of an array of strings that represent clone stable IDs, absolute occurrence IDs, or
        ID&rsquo;s in context,  depending on the version of the Product View data model. The output data will indicate
        the visible line type in the Snapshot3DVisibleLines using the uidtype. If the type is a clone stable UID chain,
        this is a &lsquo;/&rsquo; delimited string that represents a path from the root of the structure to the visible
        line of interest. This list will be obtained by processing the structure PLMXML file.
        
        The return information also provides the core files that need to be opened by the viewer (e.g. the viewFile
        PLMXML and/or the structure file PLMXML). The viewFile is used to recreate the 3D scene, and the structure file
        can be used to open the static structure for the Product View to recreate the same scene that was saved
        originally.
        
        Use cases:
        This service helps provide the viewer with information to load Product View objects. There are 3 main use cases
        for loading Product Views, each described in more detail below.
        
        
        &bull;Load Static Product View: In this case the static structure for the visible lines and their descendants
        is loaded from the Product View data model via the structure PLMXML file. This returns the Product View to the
        as saved state as far as product structure goes, but changes to part level JTs due to overwrite operations can
        still be present.
        &bull;Load dynamic Product View: In this case the structure configuration information stored in the Product
        View data model that represents the configuration of the structure when the Product View was last saved is used
        to reconfigure the structure with the same configuration rules before the scene is recreated. 
        &bull;Load dynamic Product View: In this case the current BOM window configuration established in the Structure
        Manager (SM), Multi Structure Manager (MSM), or Manufacturing Process Planner (MPP) application is the
        structure configuration used to apply the Product View on.
        
        
        Load static Product View:
        &bull;User selects one or more Product View objects (SnapShotViewData Dataset objects) in My Teamcenter and
        sends those to open in visualization. 
        &bull;The visualization client gets the list of objects to open, builds up the Snapshot3DInfoInput2 input
        structure for each Dataset, and calls the getSnapshot3DInfo operation for the Product Views. 
        &bull;The system reads the data model and returns the visible lines list, the named references for the files,
        and related objects. 
        &bull;The viewer downloads the files of interest from the Dataset (e.g. structure PLMXML file, viewFile PLMXML,
        and 3D markup layers (vpl markup layers)). 
        &bull;The viewer loads the structure PLMXML file, loads the markup layers (vpl files), and then merges in the
        viewFile PLMXML to recreate the 3D scene.
        
        Load dynamic Product View using stored configuration rules:
        &bull;User selects one or more Product View objects in My Teamcenter and sends those to open in visualization. 
        &bull;The visualization client gets the list of related objects to open, builds up the Snapshot3DInfoInput2
        structure for each Dataset, and calls the getSnapshot3DInfo operation for the Product Views. 
        &bull;The system reads the data model and returns the visible lines list, the named references, and related
        objects. 
        &bull;The viewer downloads the files of interest from the Dataset (structure PLMXML file, viewFile PLMXML, vpl
        markup layers). 
        &bull;The viewer uses the related objects information to get the top line and the configuration information for
        the structure via the VisStructureContext object, and calls the createBOMsFromRecipes operation to configure
        the structure properly. 
        &bull;The viewer uses the visible line information to expand the BOM structure and fetch BOMLine objects. If
        clone stable occurrence id chains are the type of visible lines returned, the Viewer starts at the top of the
        structure and uses the expandPSOneLevel operation to expand the structure and load it. If the absolute
        occurrences are the type of visible lines returned or the preference to prune the structure during Product View
        load is active, the viewer uses the expandPSFromOccurrenceList operation to expand the structure and fetch the
        visible BOMLine objects. 
        &bull;Once the configured structure is loaded and expanded, the viewer loads the markup layers (vpl files), and
        then merges in the viewFile PLMXML to recreate the 3D scene.
        
        Load dynamic Product View using current configuration rules:
        &bull; User sends a structure to Structure Manager (SM), Multi Structure Manager (MSM), or Manufacturing
        Process Planner (MPP) and configures the structure with revision rules, variant rules, effectivity, etc. 
        &bull; The user brings up the Product View gallery, and the system finds Product Views of interest by calling
        the gatherSnapshot3DList operation.  
        &bull; The user selects some Product Views and sends those to visualization. For standalone visualization a
        VisStructureContext is sent from the launching client with the launch file that represents the current
        configuration of the BOMWindow. For embedded visualization, the current BOMWindow configuration is already
        known.
        &bull; The visualization client gets the list of objects to open, builds up the Snapshot3DInfoInput2 structure
        for each object, and calls the getSnapshot3DInfo operation for the Product Views. 
        &bull; The system reads the data model and returns the visible lines list, the named references, and related
        objects. 
        &bull; The viewer downloads the files of interest from the Dataset (structure PLMXML file, viewFile PLMXML, vpl
        markup layers). 
        &bull; The viewer uses the visible line information to expand the BOM structure and fetch BOMLine objects if
        they are not already loaded. If clone stable occurrence ID chains are the type of visible lines returned, the
        Viewer starts at the top of the structure and uses the expandPSOneLevel operation to expand the structure and
        load it. If the absolute occurrences are the type of visible lines returned or the preference to prune the
        structure during Product View load is active, the viewer uses the expandPSFromOccurrenceList operation to
        expand the structure and fetch the visible BOMLine objects. 
        &bull; Once the configured structure is loaded and expanded, the viewer loads the markup layers (.vpl files),
        and then merges in the view File PLMXML to recreate the 3D scene.
        """
        return cls.execute_soa_method(
            method_name='getSnapshot3DInfo2',
            library='Internal-Visualization',
            service_date='2017_05',
            service_name='DataManagement',
            params={'snapshot3DInputList': snapshot3DInputList},
            response_cls=Snapshot3DInfoResponse2,
        )



from __future__ import annotations

from tcsoa.gen.Reports._2007_01.CrfReports import ReportProperties, GetReportDefinitionsResponse, GenerateReportResponse, ReportsCriteria, GenerateReportCriteria, CreateReportDefinitionResponse, GenerateReportDefintionIdsResponse
from typing import List
from tcsoa.base import TcService


class CrfReportsService(TcService):

    @classmethod
    def getReportDefinitions(cls, inputCriteria: List[ReportsCriteria]) -> GetReportDefinitionsResponse:
        """
        Retrieves a set of report definitions that meet the specified criteria.
        
        Use cases:
        Document set of user level use cases, should describe how user interacts with this operation to accomplish the
        goal.
        """
        return cls.execute_soa_method(
            method_name='getReportDefinitions',
            library='Reports',
            service_date='2007_01',
            service_name='CrfReports',
            params={'inputCriteria': inputCriteria},
            response_cls=GetReportDefinitionsResponse,
        )

    @classmethod
    def createReportDefinition(cls, reportProperties: ReportProperties) -> CreateReportDefinitionResponse:
        """
        Creates a report definition with the specified properties.
        """
        return cls.execute_soa_method(
            method_name='createReportDefinition',
            library='Reports',
            service_date='2007_01',
            service_name='CrfReports',
            params={'reportProperties': reportProperties},
            response_cls=CreateReportDefinitionResponse,
        )

    @classmethod
    def generateReport(cls, inputCriteria: GenerateReportCriteria, applicationId: str) -> GenerateReportResponse:
        """
        Generates report (Summary Report/Custom Report/Item Report) using the specified criteria and the selected
        report style sheet. The report can be displayed with the selected report style. The report style is used to
        define how to display the report result in UI to end user. The report can optionally be saved as a Dataset in
        the Teamcenter database. The generated report is saved in the FMS transient volume and an FMS ticket is
        returned.
        
        Use cases:
        User can generate one report (Summary Report/Custom Report/Item Report) by selecting one report definition and
        then inputs criteria for the report query, selects one report style sheet, and inputs the dataset name for the
        report if user would like to save the report as a dataset in Teamcenter.
        """
        return cls.execute_soa_method(
            method_name='generateReport',
            library='Reports',
            service_date='2007_01',
            service_name='CrfReports',
            params={'inputCriteria': inputCriteria, 'applicationId': applicationId},
            response_cls=GenerateReportResponse,
        )

    @classmethod
    def generateReportDefintionIds(cls, inputCriteria: List[ReportsCriteria]) -> GenerateReportDefintionIdsResponse:
        """
        Generates new report definition id(s) based upon the specified report criteria. This operation is used when
        user creates new report definition; the report definition id is generated by this operation. Multiple report
        definition id generation is not supported currently.
        
        Use cases:
        User creates a new report definition from the New Report wizard. User presses the button to generate the report
        definition id for the new report definition. This operation will be triggered to generate the report definition
        id and assign to the new report definition.
        """
        return cls.execute_soa_method(
            method_name='generateReportDefintionIds',
            library='Reports',
            service_date='2007_01',
            service_name='CrfReports',
            params={'inputCriteria': inputCriteria},
            response_cls=GenerateReportDefintionIdsResponse,
        )



from tcsoa.gen.Internal_Cae._2012_09.services import StructureManagementService as imp0
from tcsoa.gen.Internal_Cae._2011_06.services import StructureManagementService as imp1
from tcsoa.gen.Internal_Cae._2012_02.services import StructureManagementService as imp2
from tcsoa.gen.Internal_Cae._2013_05.services import StructureManagementService as imp3
from tcsoa.gen.Internal_Cae._2012_02.services import SimulationProcessManagementService as imp4
from tcsoa.gen.Internal_Cae._2013_12.services import SimulationProcessManagementService as imp5
from tcsoa.gen.Internal_Cae._2014_06.services import StructureManagementService as imp6
from tcsoa.gen.Internal_Cae._2009_10.services import SimulationProcessManagementService as imp7
from tcsoa.gen.Internal_Cae._2013_12.services import StructureManagementService as imp8
from tcsoa.base import TcService


class StructureManagementService(TcService):
    createNewModelByDM = imp0.createNewModelByDM
    executeDatamap = imp1.executeDatamap
    executeDatamap2 = imp0.executeDatamap
    executeMarkUpToDate = imp0.executeMarkUpToDate
    executeStructureMap = imp1.executeStructureMap
    generateNodeXML = imp2.generateNodeXML
    getCAEPropertyComparisonDetails = imp3.getCAEPropertyComparisonDetails
    loadSimulationDataMonitor = imp6.loadSimulationDataMonitor
    propagateCAEModelAttributes = imp8.propagateCAEModelAttributes
    refreshSimulationDataMonitor = imp6.refreshSimulationDataMonitor
    updateModelAttsByDM = imp0.updateModelAttsByDM


class SimulationProcessManagementService(TcService):
    importSimulationObjects = imp4.importSimulationObjects
    importSimulationObjects2 = imp5.importSimulationObjects2
    notifyUser = imp7.notifyUser



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, Group, Role
from typing import List
from tcsoa.gen.Administration._2007_06.Authorization import NameAuthorizationList, NameInfo
from tcsoa.base import TcService


class AuthorizationService(TcService):

    @classmethod
    def checkAuthorizationAccess(cls, user: User, group: Group, role: Role, inputNames: List[NameInfo]) -> NameAuthorizationList:
        """
        This operation can be used to get authorization access on the given applications and utilities for the given
        user, group and role combination. Rule domain specifies accessibility need to be checked on an application or
        on a utility. Valid values for the domain are "utility" and "application".  If some other string is specified
        as rule domain this operation will return error code 290006.  However accessibility check for correct domain
        names will continue. Following are the valid application names and utility names for this operation. For more
        information on authorization rules please refer to Authorization guide in Teamcenter documentation.
        
        List of application IDs:
        Organization
        Business_Modeler_IDE
        Access_Manager
        Archive_Restore
        Setup_Wizard
        Workflow_Designer
        PLMXML_Import_Export
        Project
        Subscription_Monitor
        Classification_Admin
        Report_Designer
        Application_Configuration
        EIntegrator_Admin
        Audit_Manager
        Authorization
        Schema_Editor
        Appearance_Configuration
        ADA License
        
        List of utility IDs:
        data_share
        export_recovery
        database_verify
        update_project_data
        data_sync
        dsa_util
        import_export_business_rules
        purge_invalid_subscriptions
        create_change_types
        fsc_admin
        ada_util
        attribute_export
        
        Use cases:
        To check user's accessibility.
        
        - While opening an admin application.
        - While running an admin utility
        
        """
        return cls.execute_soa_method(
            method_name='checkAuthorizationAccess',
            library='Administration',
            service_date='2008_12',
            service_name='Authorization',
            params={'user': user, 'group': group, 'role': role, 'inputNames': inputNames},
            response_cls=NameAuthorizationList,
        )



from __future__ import annotations

from tcsoa.gen.Internal.BusinessModeler._2011_06.DataModelManagement import ServerCacheResponse
from tcsoa.base import TcService


class DataModelManagementService(TcService):

    @classmethod
    def updateClientMetaCache(cls, option: int) -> str:
        """
        update client meta cache basd on delta xml during BMIDE hot deploy.
        """
        return cls.execute_soa_method(
            method_name='updateClientMetaCache',
            library='Internal-BusinessModeler',
            service_date='2011_06',
            service_name='DataModelManagement',
            params={'option': option},
            response_cls=str,
        )

    @classmethod
    def updateServerMetaCache(cls, option: int) -> ServerCacheResponse:
        """
        This Teamcenter Service generates or deletes the metadata cache based on the value passed in the input
        argument. If the argument (option=0) is defined for creating the metadata cache, then the metadata cache is
        created and saved into a binary file and persisted in the database in the form of a dataset. If the argument
        (option=1) is defined for deleting the metadata cache, then the existing metadata cache dataset in the database
        is deleted. The Teamcenter servers starting up after this point will run in the local cache mode. NOTE: This
        operation is invoked during live deployment from BMIDE
        
        Use cases:
        Use case 1: Update the metadata cache to improve Teamcenter server memory footprint and performance after live
        deployment of data model changes such as Business Objects, Properties and Constants
        
        Turn on the option by setting it to 0, and deploy the metadata changes to the database. The metadata cache
        binary file will be generated and the cache dataset will be updated in the database. The Teamcenter server(s)
        will pick up the updated cache. 
        
        Use case 2: Delete the metadata cache after live deployment of data model changes such as Business Objects,
        Properties and Constants
        
        Turn off the option by setting it to 1, and deploy the metadata changes to the database. The metadata cache
        dataset will be deleted from the database if it exists. Teamcenter server(s) functioning at the moment will
        continue to use the stale cache i.e., the deployed metadata will not be reflected in these caches. Teamcenter
        server(s) starting up after this stage will run in the local cache mode.
        
        NOTE: In order to improve live deploy performance, typically, interim deployments will be done with the option
        set to 1. Finally, when user is done with metadata changes, the option can be set to 0 so that cache is
        generated on the server which improves Teamcenter performance.
        """
        return cls.execute_soa_method(
            method_name='updateServerMetaCache',
            library='Internal-BusinessModeler',
            service_date='2011_06',
            service_name='DataModelManagement',
            params={'option': option},
            response_cls=ServerCacheResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import DispatcherServiceConfig
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ConfigInput(TcBaseObj):
    """
    The 'ConfigInput' structure is used to filter source dataset type, derived dataset type, and service available for
    DispatcherServiceConfig business objects query.
    
    :var sourceDatasetTypeName: The input source dataset type name.  Set to empty string (use double quotes to specify
    empty string) to query all source dataset type name.
    :var derivedDatasetTypeName: The input derived dataset type name. Set to empty string (use double quotes to specify
    empty string) to query all derived dataset type name.
    :var serviceAvailable: The input service available state.  Set to 0 means for false state, set to 1 means for true
    state, and set to negative/minus 1 means to ignore service available state.
    """
    sourceDatasetTypeName: str = ''
    derivedDatasetTypeName: str = ''
    serviceAvailable: int = 0


@dataclass
class ConfigOutput(TcBaseObj):
    """
    This structure contains the output list DispatcherServiceConfig business objects and the 'Service Data'.
    
    :var configOutputData: List of DispatcherServiceConfig business objects, one for each 'ConfigInput' in the list.
    :var svcData: The' Service Data'. Partial errors and failures are updated and returned through this object.
    """
    configOutputData: List[ConfigOutputData] = ()
    svcData: ServiceData = None


@dataclass
class ConfigOutputData(TcBaseObj):
    """
    This structure contains the output list of DispatcherServiceConfig business objects, one for each 'ConfigInput' in
    the list.
    
    :var svcConfig: List of DispatcherServiceConfig business objects.  Can be empty if no object found.
    """
    svcConfig: List[DispatcherServiceConfig] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Rdv._2007_06.VariantManagement import GetVariabilityInfoResponse
from typing import List
from tcsoa.base import TcService


class VariantManagementService(TcService):

    @classmethod
    def getVariabilityInfo(cls, inputs: List[BusinessObject]) -> GetVariabilityInfoResponse:
        """
        Gets the Variability information assigned to the list of object supplied as input to this operation through the
        'inputs' list. The input objects could be any of the following
        -     Product Context :  (Product Item or Item Revision)
        -     Architecture Breakdown :  An object of type Architecture or its subtypes.
        -     Architecture Breakdown Element : Any leaf node or intermediate bomline which is of type Architecture or
        its subtypes. The Architecture Breakdown Element should have a valid generic_component_id which should be
        unique amongst all ABEs of the same type of Architecture.
        
        
        
        The operation would not succeed if any other objects are specified as inputs to this operation.
        
        Use cases:
        The 'getVariabilityInfo' operation is called when user wants to fetch the Variability information on Product
        Context or Architecture Breakdown or Architecture Breakdown Element. User can specify the input as a list of
        Product Context or Architecture Breakdown or Architecture Breakdown Element.
        """
        return cls.execute_soa_method(
            method_name='getVariabilityInfo',
            library='Internal-Rdv',
            service_date='2007_06',
            service_name='VariantManagement',
            params={'inputs': inputs},
            response_cls=GetVariabilityInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2017_11.IRM import AccessorsInfoResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.base import TcService


class IRMService(TcService):

    @classmethod
    def getAccessorsInfo(cls, accessorObjects: List[BusinessObject]) -> AccessorsInfoResponse:
        """
        This operation can be used to get the name and type of the given accessor object. This name and type
        information is used by the Access Manager application to populate the accessors on the Access Control List.
        
        Use cases:
        Clients like RAC call this operation to get the name and type of the accessors. Access Manager application in
        RAC have a tabular representation of the accessors. Using this table user can grant or revoke the access for a
        particular privilege against a particular accessor. To render each such accessor on this Access Control List
        table RAC UI needs information like accessor name and accessor type. This information will be required for each
        accessor specified on the table. This operation will be used to get this information in bulk and render on the
        UI through UI component.
        """
        return cls.execute_soa_method(
            method_name='getAccessorsInfo',
            library='Internal-Administration',
            service_date='2017_11',
            service_name='IRM',
            params={'accessorObjects': accessorObjects},
            response_cls=AccessorsInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Rdv._2013_05.ContextManagement import ContextObjectResponse
from typing import List
from tcsoa.base import TcService


class ContextManagementService(TcService):

    @classmethod
    def getRelatedObjectsInContext(cls, objectList: List[BusinessObject]) -> ContextObjectResponse:
        """
        This operation finds all the business objects that are within the current active context of the list of objects
        sent as input. The objects within context are objects of the types specified below that are related to the
        input object either through reference or relation.
        - Items
        - Processes
        - ChangeRequestRevision
        - StructureContext
        - AppearanceGroup
        
        
        
        
        The RDV module maintains a state that defines the currently active context. A combination of one or more of the
        following objects defines the current active context.
        - WorkpartItem
        - Process
        - ChangeRequestRevision
        - ConfigurationContext
        - StructureContext
        - AppearanceGroup
        
        
        
        
        Every time this operation is invoked, the objects in the current context are refreshed.
        This operation is mainly used for adding work parts, EngChange revisions and processes to the Design Context
        application.
        The following criteria is used to determine the objects related to the inputs:
        
        1) Item objects that are attached with the ChangeRequestRevision, Process objects through the relation type
        mentioned in the preferences listed below.
        
        'PortalDesignContextProcess.WorkPartOrChangeAttachmentTypes'
        'PortalDesignContextECObject.WorkPartRelationships'
        
        
        2) Item objects that are associated with the EPM job where the input object is also a part of the EPM job.
        
        3) Item objects that are part of the same ConfigurationContext object, StructureContext object or an
        AppearanceGroup object as that of the input objects.
        
        Use cases:
        Use Case 1: If user enters an item ID in the WorkParts text field in DesignContext application. The Design
        Context application uses this operation to display the item revision in the WorkParts list, and any changes or
        processes that reference the item revision in the EngChange Revision and Processes lists.
        
        Use Case 2: If user enters a change ID in the EngChange Revision text field DesignContext application. The
        DesignContext application uses this operation to add the change object to the EngChange Revision list, and any
        item revisions or processes referenced by the change object are displayed in the WorkParts and Processes lists.
        If the change references revisions of Product Items, Teamcenter automatically adds them to the Selected Product
        Context list.
        
        Use Case 3: If user enters a process name in Processes test field DesignContext application. The DesignContext
        application uses this operation to add the process object to the Processes list, and any objects targeted by
        the process are displayed in the WorkParts and EngChange lists.
        
        The items in the WorkPart lists can then be selected by the user for carrying out the design validation
        analysis.
        """
        return cls.execute_soa_method(
            method_name='getRelatedObjectsInContext',
            library='Rdv',
            service_date='2013_05',
            service_name='ContextManagement',
            params={'objectList': objectList},
            response_cls=ContextObjectResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetLicenseDetailsResponse(TcBaseObj):
    """
    'GetLicenseDetailsResponse' represents the details of ADA_License business objects.
    
    :var licDetails: Structures containing the properties of ADA_License business objects.
    :var serviceData: Contains partial errors encountered while fetching properties for ADA_License business objects.
    The ADA_License business objects for which properties are fetched successfully are added to the Plain list.
    """
    licDetails: List[LicenseDetails] = ()
    serviceData: ServiceData = None


@dataclass
class LicenseDetails(TcBaseObj):
    """
    'LicenseDetails' structure represents all the details of an ADA License object.
    
    :var licenseType: The type of ADA License object. The type can be ITAR_License, Exclude_License, or IP_License.
    :var licenseId: The unique ID of the license. This is string with a maximum of 128 bytes.
    :var expiryDate: The expiry date for the license, after which the license cannot be attached to WorkspaceObject
    business objects and ceases to grant the access to users/groups listed on the license. A NULL date specifies the
    license will never expire.
    :var lockDate: The freeze date for the license, after which the license cannot be attached to WorkspaceObject
    business objects. A NULL date specifies the license is not locked.
    :var licenseReason: A reason for the license to exist. This parameter can be a maximum of 128 bytes. This is an
    optional parameter and may have value as a blank string
    :var qualifyingCfr: The qualifying Code of Federal Regulations (CFR) for ITAR licenses. This is not applicable for
    IP and Exclude licenses. This parameter can be a maxiumum of 80 bytes. This is an optional parameter and  may have
    value as a blank string.
    :var users: The list of users associated with a license identified by 'licenseId'. When the license is attached to
    a classified WorkspaceObject, the users listed on the license will get access to the WorkspaceObject, based on the
    access rules.
    :var groups: List of names of groups associated with the license identified by 'licenseId'. For subgroups, the full
    names should be specified. When the license is attached to a classified WorkspaceObject, the users from the
    groups/subgroups listed on the license will get access to the workspace object, based on the access rules. This
    parameter represents an array of group name strings of upto 256 bytes in size.
    """
    licenseType: str = ''
    licenseId: str = ''
    expiryDate: datetime = None
    lockDate: datetime = None
    licenseReason: str = ''
    qualifyingCfr: str = ''
    users: List[str] = ()
    groups: List[str] = ()


@dataclass
class LicenseInput(TcBaseObj):
    """
    'LicenseInput' represents a list of license IDs of ADA licenses, associated WorkspaceObject business objects, and
    applicable authorizing paragraph information (valid for ITAR licenses only).
    
    :var selectedLicenses: List of license IDs of ADA licenses. These are strings with a maximum of 128 bytes size.
    :var objects: List of WorkspaceObject business objects associated with the 'selectedLicenses'.
    :var eadParagraph: List of authorizing paragraphs for the licenses being attached to WorkspaceObject business
    objects. These are strings with a maximum of 80 bytes size. The size of 'eadParagraph' vector should match the size
    of the 'selectedLicenses' (each entry in 'eadParagraph' maps to corresponding entry in 'selectedLicenses'). If a
    paragraph entry is not applicable for a specific license (paragraph entries are applicable only for licenses of
    ITAR type), then that entry can be left blank. System will ignore any paragraph entry if it is not applicable for a
    license to be attached. This is an optional parameter
    """
    selectedLicenses: List[str] = ()
    objects: List[BusinessObject] = ()
    eadParagraph: List[str] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class OtSearchInfo(TcBaseObj):
    """
    It specifies criteria for searching briefcase transactions that include OT.
    
    :var transactionType: Type of briefcase transaction i.e. briefcase is exported or imported.
    :var siteId: Id of the site to which briefcase is exported to or is imported from.
    :var startDate: Earliest date and time breifcase transaction must have occured.
    :var endDate: Latest date and time briefcase transaction must have occured.
    :var briefcaseName: Name of the briefcase file.
    """
    transactionType: str = ''
    siteId: int = 0
    startDate: datetime = None
    endDate: datetime = None
    briefcaseName: str = ''


@dataclass
class OtTransactionInfo(TcBaseObj):
    """
    It contains information of a briefcase transaction (that include OT) like transaction id, transaction type, site
    id, user id, transaction date and briefcase name.
    
    :var transactionId: Identifier of the briefcase transaction that includes OT.
    :var transactionType: Type specifying briefcase is exported/imported.
    :var siteId: Current owning site of the briefcase.
    :var userId: Id of the user who exported/imported the briefcase.
    :var transactionDate: Date on which transaction occurred.
    :var briefcaseName: Name of the briefcase file.
    """
    transactionId: str = ''
    transactionType: str = ''
    siteId: int = 0
    userId: str = ''
    transactionDate: datetime = None
    briefcaseName: str = ''


@dataclass
class OtTransactionResponse(TcBaseObj):
    """
    Response for findOtTransactions operation. It contains list of briefcase transactions (that include OT) and errors
    if any.
    
    :var otTransactions: List of OtTransactionInfo objects.
    :var serviceData: Service data contains the errors if the operation fails.
    """
    otTransactions: List[OtTransactionInfo] = ()
    serviceData: ServiceData = None


@dataclass
class RecoverOwnershipResponse(TcBaseObj):
    """
    It holds the FMS ticket of report which has information about the status of recover ownership operation and errors
    if any.
    
    :var reportFmsTicket: FMS ticket of the report file listing the details of all objects for which ownership is
    recovered. The FMS ticket can be used to download the file from server to client.
    :var serviceData: Service data contains the errors if the operation fails.
    """
    reportFmsTicket: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class UserConsentStatement(TcBaseObj):
    """
    Output strucutre of getUserConsentStatement operation.
    
    :var consentStatement: User consent statement for the locale the user is logged in.
    :var serviceData: Service data with the partial error information.
    """
    consentStatement: str = ''
    serviceData: ServiceData = None
